package com.nationrel.modules.fta.controller;

import com.nationrel.modules.fta.entity.FtaAnalysis;
import com.nationrel.modules.fta.vo.FtaAnalysisTreeVo;
import com.nationrel.modules.fta.general.common.ServerResponse;
import com.nationrel.modules.fta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.fta.general.domain.enums.common.AuthorityProgramExceptionEnum;
import com.nationrel.modules.fta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.fta.handler.GlobalExceptionHandler;
import com.nationrel.modules.fta.service.FtaAnalysisService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

@Tag(name = "FTA-分析")
@RestController
@RequestMapping("/fta/analysis")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FtaAnalysisController {

    private final FtaAnalysisService analysisService;

    @Operation(summary = "分析树")
    @GetMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "projectId", description = "分析Id", in = ParameterIn.QUERY)
    })
    public ServerResponse<List<FtaAnalysisTreeVo>> analysisList(HttpServletRequest request, @RequestParam(required = false) String projectId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return analysisService.listByPage(userInfo, projectId);
    }

    @Operation(summary = "分析创建")
    @PostMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> createAnalysis(HttpServletRequest request, @Valid @RequestBody FtaAnalysis entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(entity.getAnalysisName())) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析名称不可以为空, userInfo.getLanguageKey());
        }
        return analysisService.addAnalysis(userInfo, entity);
    }

    @Operation(summary = "分析修改")
    @PutMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> updateAnalysis(HttpServletRequest request, @Valid @RequestBody FtaAnalysis entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(entity.getAnalysisId())) {
            return ServerResponse.createByErrorMessage(AuthorityProgramExceptionEnum
                    .getLanguageMessageByKey(AuthorityProgramExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return analysisService.updateAnalysis(userInfo, entity);
    }

    @Operation(summary = "分析名称修改")
    @PostMapping("/name")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> updateAnalysisName(HttpServletRequest request, @RequestBody FtaAnalysis entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(entity.getAnalysisId()) || StringUtils.isBlank(entity.getAnalysisName())) {
            return ServerResponse.createByErrorMessage(AuthorityProgramExceptionEnum
                    .getLanguageMessageByKey(AuthorityProgramExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return analysisService.updateAnalysisName(userInfo, entity);
    }

    @Operation(summary = "分析删除")
    @DeleteMapping("/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> deleteAnalysis(HttpServletRequest request, @PathVariable(value = "analysisId") String analysisId) {
        if (StringUtils.isBlank(analysisId)) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        return analysisService.deleteAnalysis(analysisId, GlobalExceptionHandler.parseUserInfo());
    }

    @Operation(summary = "分析复制")
    @PostMapping("/copy/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> copyAnalysis(HttpServletRequest request, @PathVariable(value = "analysisId") String analysisId) {
        if (StringUtils.isBlank(analysisId)) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        return analysisService.copyAnalysis(analysisId, GlobalExceptionHandler.parseUserInfo());
    }

    @Operation(summary = "分析导入")
    @PostMapping("/import")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> importAnalysis(
            HttpServletRequest request,
            @RequestParam("file") MultipartFile file,
            @RequestParam("projectId") String projectId) {
        if (StringUtils.isBlank(projectId) || file == null || file.isEmpty()) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.请检查数据, GlobalExceptionHandler.parseLk());
        }
        return analysisService.importAnalysis(projectId, file, GlobalExceptionHandler.parseUserInfo());
    }

    @Operation(summary = "分析导出")
    @GetMapping("/export")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public void exportAnalysis(
            HttpServletResponse response,
            HttpServletRequest request,
            @RequestParam("analysisId") String analysisId) {
        analysisService.exportAnalysis(response, analysisId, GlobalExceptionHandler.parseUserInfo());
    }

    @Operation(summary = "节点层级和连线顺序修复")
    @GetMapping("/repairDiagramData/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> repairDiagramData(HttpServletRequest request, @PathVariable(value = "analysisId") String analysisId) {
        if (StringUtils.isBlank(analysisId)) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        return analysisService.repairDiagramData(analysisId, GlobalExceptionHandler.parseUserInfo());
    }
}
