package com.nationrel.modules.fta.controller;


import com.nationrel.modules.fta.entity.FtaAnalysisCalcResult;
import com.nationrel.modules.fta.vo.FtaEventVo;
import com.nationrel.modules.fta.vo.FtaImageVo;
import com.nationrel.modules.fta.entity.FtaNode;
import com.nationrel.modules.fta.general.common.ResponseStatus;
import com.nationrel.modules.fta.general.common.ServerResponse;
import com.nationrel.modules.fta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaNodeMoveEnum;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaParamDataExceptionEnum;
import com.nationrel.modules.fta.general.exception.DescribeException;
import com.nationrel.modules.fta.handler.GlobalExceptionHandler;
import com.nationrel.modules.fta.service.FtaDiagramDataService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "FTA-分析框图")
@RestController
@Validated
@RequestMapping("/fta/data")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FtaAnalysisDataController {

    @Autowired
    private FtaDiagramDataService diagramDataService;

    @Operation(summary = "框图数据")
    @GetMapping("/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "ftaId", description = "数据ID", in = ParameterIn.QUERY)
    })
    public ServerResponse getNodesData(HttpServletRequest request, @PathVariable(value = "analysisId") String analysisId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isEmpty(analysisId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.getNodesData(analysisId);
    }

    @Operation(summary = "添加节点")
    @PostMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse insert(HttpServletRequest request, @RequestBody FtaEventVo entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (entity == null || entity.getSourceId() == null) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.insert(entity, userInfo);
    }

    @Operation(summary = "更新节点")
    @PutMapping("update")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse update(HttpServletRequest request, @Valid @RequestBody FtaNode entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (entity == null) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.update(userInfo, entity, true);
    }

    @Operation(summary = "批量更新")
    @PutMapping("/nodes")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> updates(HttpServletRequest request, @Valid @RequestBody List<FtaNode> list) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (CollectionUtils.isEmpty(list)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.update(list, userInfo);
    }

    @Operation(summary = "删除数据")
    @DeleteMapping("/{id}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse delete(HttpServletRequest request, @PathVariable(value = "id") String id) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(id)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.delete(id, userInfo);
    }

    @Operation(summary = "获取可添加的重复事件数据")
    @GetMapping("/getMasterNodes/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getMasterNodes(HttpServletRequest request, @PathVariable(value = "analysisId") String analysisId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(analysisId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.getMasterNodes(analysisId, userInfo);
    }

    @Operation(summary = "获取事件编码")
    @GetMapping("/getEventCode/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getEventCode(HttpServletRequest request, @PathVariable(value = "analysisId") String analysisId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(analysisId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.getEventCode(analysisId, userInfo);
    }

    @Operation(summary = "节点复制")
    @GetMapping("/nodeCopy/{sourceId}/{targetId}/{isPasteToMirror}/{isPasteToMirrorTransfer}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse nodeCopy(
            HttpServletRequest request,
            @PathVariable(value = "sourceId") String sourceId,
            @PathVariable(value = "targetId") String targetId,
            @PathVariable(value = "isPasteToMirror") boolean isPasteToMirror,
            @PathVariable(value = "isPasteToMirrorTransfer") boolean isPasteToMirrorTransfer) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(sourceId) || StringUtils.isBlank(targetId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        if (isPasteToMirrorTransfer) {
            return diagramDataService.nodeCopyTransfer(userInfo, sourceId, targetId);
        }
        return diagramDataService.nodeCopy(userInfo, sourceId, targetId, isPasteToMirror);
    }

    @Operation(summary = "节点剪切")
    @GetMapping("/nodeCut/{sourceId}/{targetId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse nodeCut(
            HttpServletRequest request,
            @PathVariable(value = "sourceId") String sourceId,
            @PathVariable(value = "targetId") String targetId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(sourceId) || StringUtils.isBlank(targetId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        } else if (sourceId.equals(targetId)) {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请勿将一个事件剪切到该事件的子集中, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
        return diagramDataService.nodeCut(userInfo, sourceId, targetId, true);
    }

    @Operation(summary = "计算最小割集及失效率", description = "计算最小割集及失效率")
    @PostMapping("/generateSysEquation/{ftaId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse generateSysEquation(HttpServletRequest request, @PathVariable(value = "ftaId") String ftaId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isEmpty(ftaId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.generateSysEquationWang(userInfo, ftaId);
    }

    @Operation(summary = " 验证分析是否满足绘图条件", description = " 验证分析是否满足绘图条件，并返回可绘制的节点数据")
    @GetMapping("/validPlot/{ftaId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse validPlot(HttpServletRequest request, @PathVariable(value = "ftaId") String ftaId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isEmpty(ftaId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.validPlot(ftaId, userInfo);
    }

    @Operation(summary = "获取历史系统方程", description = "获取历史系统方程")
    @GetMapping("/getSysEquation/{ftaId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "ftaId", description = "数据ID", in = ParameterIn.QUERY)
    })
    public ServerResponse<FtaAnalysisCalcResult> getSysEquation(HttpServletRequest request, @PathVariable(value = "ftaId") String ftaId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isEmpty(ftaId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.getSysEquation(ftaId);
    }

    @Operation(summary = "更新节点展开收缩")
    @PutMapping("updateLeaf/{nodeId}/{leaf}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> updateLeaf(HttpServletRequest request, @PathVariable(value = "nodeId") String nodeId, @PathVariable(value = "leaf") Boolean leaf) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(nodeId) || leaf == null) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.updateLeaf(nodeId, leaf);
    }

    @Operation(summary = "更新线的顺序字段")
    @PutMapping("updateOrder/{sourceId}/{targetId}/{moveDirection}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse updateOrder(
            HttpServletRequest request,
            @PathVariable(value = "sourceId") String sourceId,
            @PathVariable(value = "targetId") String targetId,
            @PathVariable(value = "moveDirection") Integer moveDirection) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(sourceId) || StringUtils.isBlank(targetId) || StringUtils.isBlank(FtaNodeMoveEnum.getNameByIndex(moveDirection))) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.updateOrder(sourceId, targetId, moveDirection, userInfo);
    }

    @Operation(summary = "回退（Ctrl+Z）")
    @GetMapping("/rollback/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "ftaId", description = "数据ID", in = ParameterIn.QUERY)
    })
    public ServerResponse rollback(
            HttpServletRequest request,
            @PathVariable(value = "analysisId") String analysisId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isEmpty(analysisId)) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.rollback(userInfo, analysisId);
    }

//    @Operation(summary = "计算器快速计算数据", notes = "快速计算数据")
//    @PostMapping("/fastCalcResult")
//    @Parameters(value = {
//            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
//    })
//    public ServerResponse fastCalcResult(HttpServletRequest request, @RequestBody FtaFastCalcVo fastCalcVo) {
//        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
//        if (fastCalcVo == null) {
//            return ServerResponse.createByErrorMessage(AuthorityProgramExceptionEnum.getLanguageMessageByKey(AuthorityProgramExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
//        }
//        return diagramDataService.fastCalcResult(userInfo, fastCalcVo);
//    }

    @Operation(summary = "图像信息", description = "图像信息")
    @PostMapping("/image")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse image(HttpServletRequest request, @RequestBody FtaImageVo imageVo) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (imageVo == null) {
            return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        return diagramDataService.image(userInfo, imageVo);
    }
}
