// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.fta.service;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.nationrel.modules.fta.bo.FtaImportBo;
import com.nationrel.modules.fta.constant.FtaConstants;
import com.nationrel.modules.fta.entity.*;
import com.nationrel.modules.fta.general.common.ResponseStatus;
import com.nationrel.modules.fta.general.common.ServerResponse;
import com.nationrel.modules.fta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaDistributingTypeCodeEnum;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaNodeEnum;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaParamDataExceptionEnum;
import com.nationrel.modules.fta.general.exception.DescribeException;
import com.nationrel.modules.fta.general.util.common.BasicDataUtil;
import com.nationrel.modules.fta.general.util.common.UUIDUtil;
import com.nationrel.modules.fta.mapper.FtaAnalysisMapper;
import com.nationrel.modules.fta.mapper.FtaNodeMapper;
import com.nationrel.modules.fta.mapper.FtaProjectMapper;
import com.nationrel.modules.fta.util.JDBC2Util;
import com.nationrel.modules.fta.vo.FtaAnalysisTreeVo;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FtaAnalysisService {

    @Value("${spring.datasource.dynamic.datasource.master.url}")
    private String mysqlUrl;
    @Value("${spring.datasource.dynamic.datasource.master.username}")
    private String mysqlUserName;
    @Value("${spring.datasource.dynamic.datasource.master.password}")
    private String mysqlPassword;
    private final FtaProjectMapper projectMapper;
    private final FtaAnalysisMapper analysisMapper;
    private final FtaNodeMapper nodeMapper;
    private final FtaDiagramDataService diagramDataService;
    private final FtaNodeService nodeService;

    /**
     * 分页查询
     *
     * @param userInfo
     * @param ftaId
     * @return
     */
    public ServerResponse<List<FtaAnalysisTreeVo>> listByPage(HeaderUserInfo userInfo, String ftaId) {
        List<FtaAnalysis> analyses = analysisMapper.selectList(new LambdaQueryWrapper<FtaAnalysis>()
                .eq(FtaAnalysis::getCreateBy, userInfo.getUserName())
                .eq(FtaAnalysis::getProjectId, ftaId));
        List<FtaAnalysisTreeVo> result = new ArrayList<>();
        analyses.forEach(analysis -> {
            // 组装每个顶分析的子集
            if (StringUtils.isBlank(analysis.getFatherId())) {
                FtaAnalysisTreeVo analysisTree = FtaAnalysisTreeVo.builder()
                        .id(analysis.getAnalysisId())
                        .projectId(analysis.getProjectId())
                        .analysisId(analysis.getAnalysisId())
                        .label(analysis.getAnalysisName())
                        .showInput(false)
                        .level(analysis.getLevel())
                        .viewType(analysis.getViewType())
                        .insertTime(analysis.getCreateTime())
                        .build();
                getFtaAnalysisTree(analysisTree, analyses, analysisTree.getId());
                result.add(analysisTree);
            }
        });
        List<FtaAnalysisTreeVo> vos = result.stream().sorted(Comparator.comparing(FtaAnalysisTreeVo::getInsertTime).reversed()).collect(Collectors.toList());
        return ServerResponse.createBySuccess(vos);
    }

    public FtaAnalysisTreeVo getFtaAnalysisTree(FtaAnalysisTreeVo ftaAnalysisTreeVo, List<FtaAnalysis> allAnalysis, String fatherId) {
        ArrayList<FtaAnalysisTreeVo> analysisTrees = new ArrayList<>();
        allAnalysis.forEach(analysis -> {
            if (StringUtils.isNotBlank(analysis.getFatherId()) && analysis.getFatherId().equals(fatherId)) {
                FtaAnalysisTreeVo analysisTree = FtaAnalysisTreeVo.builder()
                        .id(analysis.getAnalysisId())
                        .projectId(analysis.getProjectId())
                        .analysisId(analysis.getAnalysisId())
                        .label(analysis.getAnalysisName())
                        .showInput(false)
                        .level(analysis.getLevel())
                        .viewType(analysis.getViewType())
                        .build();
                getFtaAnalysisTree(analysisTree, allAnalysis, analysisTree.getId());
                analysisTrees.add(analysisTree);
            }
        });
        if (!CollectionUtils.isEmpty(analysisTrees)) {
            ftaAnalysisTreeVo.setChildren(analysisTrees);
        }
        return ftaAnalysisTreeVo;
    }

    /**
     * 添加分析数据信息
     *
     * @param userInfo
     * @param insert
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> addAnalysis(HeaderUserInfo userInfo, FtaAnalysis insert) {

        String analysisId = BasicDataUtil.uuidGenerator();
        insert.setAnalysisId(analysisId);
        insert.setCreateTime(new Date());
        insert.setCreateBy(userInfo.getUserName());
        insert.setUpdateTime(new Date());
        insert.setUpdateBy(userInfo.getUserName());
        insert.setDelFlag(FtaConstants.DEFAULT_DEL_FLAG);
        analysisMapper.insert(insert);
        // 添加分析时，需要添加默认数据
        diagramDataService.initDiagramData(analysisId, insert.getAnalysisName(), null);
        return ServerResponse.createBySuccess();
    }

    /**
     * 更新数据
     *
     * @param userInfo
     * @param entity
     * @return
     */
    public ServerResponse<String> updateAnalysis(HeaderUserInfo userInfo, FtaAnalysis entity) {
        entity.setUpdateTime(new Date());
        entity.setUpdateBy(userInfo.getUserName());
        analysisMapper.updateById(entity);
        FtaProjectService.updateUpdateInfo(userInfo, entity.getProjectId(), projectMapper);
        return ServerResponse.createBySuccess();
    }

    /**
     * 更新分析名称
     *
     * @param userInfo
     * @param entity
     * @return
     */
    public ServerResponse<String> updateAnalysisName(HeaderUserInfo userInfo, FtaAnalysis entity) {
        FtaAnalysis analysis = analysisMapper.selectById(entity.getAnalysisId());
        analysis.setAnalysisName(entity.getAnalysisName());
        analysis.setUpdateTime(new Date());
        analysis.setUpdateBy(userInfo.getUserName());
        analysisMapper.updateById(analysis);
        FtaProjectService.updateUpdateInfo(userInfo, analysis.getProjectId(), projectMapper);
        return ServerResponse.createBySuccess();
    }

    /**
     * 删除分析
     *
     * @param analysisId
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> deleteAnalysis(String analysisId, HeaderUserInfo userInfo) {
        // 判断删除的是否是转移门
        String nodeId = nodeMapper.selectByChildrenAnalysisId(analysisId);
        if (StringUtils.isNotBlank(nodeId)) {
            // 删除的是转移门，转为删除转移门父级节点
            diagramDataService.delete(nodeId, userInfo);
            return ServerResponse.createBySuccess();
        }
        FtaAnalysis analysis = analysisMapper.selectById(analysisId);
        // 删除该分析及其所有子分析
        List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(analysis.getProjectId(), userInfo.getUserId());
        ArrayList<String> analysisIds = new ArrayList<>();
        getChildrenAnalysisIds(analysisId, analysisIds, allAnalysis);
        analysisIds.add(analysisId);
        analysisMapper.deleteBatchIds(analysisIds);
        clearCalculateHistory(analysis, allAnalysis);
        return ServerResponse.createBySuccess();
    }

    /**
     * 复制整个分析
     *
     * @param analysisId
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> copyAnalysis(String analysisId, HeaderUserInfo userInfo) {
        FtaAnalysis analysis = analysisMapper.selectById(analysisId);
        // 复制节点信息
        // 找到该分析中的转移门
        List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(analysis.getProjectId(), userInfo.getUserName());
        List<FtaNode> nodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getAnalysisId, analysisId)
                .eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex())
                .isNotNull(FtaNode::getChildrenAnalysisId));
        // 获取需要更新的分析id及子树id
        ArrayList<String> analysisIds = new ArrayList<>();
        analysisIds.add(analysisId);
        // 找到该分析关联的子分析
        nodes.forEach(node -> {
            analysisIds.add(node.getChildrenAnalysisId());
            getChildrenAnalysisIds(node.getChildrenAnalysisId(), analysisIds, allAnalysis);
        });
        //批量复制分析及该分析下的节点连线数据
        copyAnalysisAndNodes(analysisIds, userInfo);
        return ServerResponse.createBySuccess();
    }

    /**
     * 根据analysisId查询有关联的所有父子分析的Id
     *
     * @param analysisId
     * @param analysisIds
     * @param allAnalysis
     */
    static String getAnalysisIds(String analysisId, List<String> analysisIds, List<FtaAnalysis> allAnalysis) {
        ArrayList<String> childrenIds = new ArrayList<>();
        ArrayList<String> fatherIds = new ArrayList<>();
        // 找到所有父分析id
        getFatherAnalysisIds(analysisId, fatherIds, allAnalysis);
        // 如果父分析id为空，则该分析为最高级分析id
        if (CollectionUtils.isEmpty(fatherIds)) {
            fatherIds.add(analysisId);
        }
        // 然后再找该最高级分析id包含的所有子分析id
        allAnalysis.forEach(analysis -> {
            if ((fatherIds.get(fatherIds.size() - 1)).equals(analysis.getAnalysisId())) {
                getChildrenAnalysisIds(analysis.getAnalysisId(), childrenIds, allAnalysis);
            }
        });
        analysisIds.add(fatherIds.get(fatherIds.size() - 1));
        analysisIds.addAll(childrenIds);
        return fatherIds.get(fatherIds.size() - 1);
    }

    /**
     * 逐层查询该分析的所有子分析Id（不包含该分析id）
     *
     * @param analysisId  fatherId
     * @param analysisIds
     * @param allAnalysis
     */
    static void getChildrenAnalysisIds(String analysisId, List<String> analysisIds, List<FtaAnalysis> allAnalysis) {
        allAnalysis.forEach(analysis -> {
            if (analysisId.equals(analysis.getFatherId())) {
                analysisIds.add(analysis.getAnalysisId());
                getChildrenAnalysisIds(analysis.getAnalysisId(), analysisIds, allAnalysis);
            }
        });
    }

    /**
     * 逐层查询该分析的所有父分析Id（不包含该分析id）
     *
     * @param analysisId
     * @param analysisIds
     * @param allAnalysis
     */
    static void getFatherAnalysisIds(String analysisId, List<String> analysisIds, List<FtaAnalysis> allAnalysis) {
        allAnalysis.forEach(analysis -> {
            if (StringUtils.isNotBlank(analysis.getFatherId()) && analysisId.equals(analysis.getAnalysisId())) {
                analysisIds.add(analysis.getFatherId());
                getFatherAnalysisIds(analysis.getFatherId(), analysisIds, allAnalysis);
            }
        });
    }

    /**
     * 根据分析id，批量复制分析
     *
     * @param analysisIds
     */
    @Transactional(rollbackFor = Exception.class)
    public void copyAnalysisAndNodes(List<String> analysisIds, HeaderUserInfo userInfo) {
        // 需要更新的所有分析
        HashMap<String, String> analysisIdMap = new HashMap<>();
        List<FtaAnalysis> inserts = analysisMapper.selectList(new LambdaQueryWrapper<FtaAnalysis>()
                .in(FtaAnalysis::getAnalysisId, analysisIds));
        inserts = inserts.stream().map(analysis -> {
            String copyAnalysisId = BasicDataUtil.uuidGenerator();
            if (null != analysis.getLevel() && 1 == analysis.getLevel()) {
                analysis.setAnalysisName(analysis.getAnalysisName() + "_copy");
            }
            analysisIdMap.put(analysis.getAnalysisId(), copyAnalysisId);
            analysis.setAnalysisId(copyAnalysisId);
            return analysis;
        }).collect(Collectors.toList());
        inserts.forEach(analysis -> {
            analysis.setFatherId(StringUtils.isNotBlank(analysis.getFatherId()) ? analysisIdMap.get(analysis.getFatherId()) : StringUtils.EMPTY);
        });
        JDBC2Util.batchAnalysisInsert(inserts, mysqlUrl, mysqlUserName, mysqlPassword, userInfo.getLanguageKey());
        // 需要更新的节点和连线
        List<FtaNode> allNodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>().in(FtaNode::getAnalysisId, analysisIds));
        HashMap<String, String> oldNodeIdAndNewMap = new HashMap<>();
        // 大数据量批量插入
        ArrayList<FtaNode> insertNodes = new ArrayList<>();
        allNodes.forEach(item -> {
            item.setAnalysisId(analysisIdMap.get(item.getAnalysisId()));
            item.setChildrenAnalysisId(StringUtils.isNotBlank(item.getChildrenAnalysisId()) ? analysisIdMap.get(item.getChildrenAnalysisId()) : StringUtils.EMPTY);
            String oldId = item.getId();
            item.setId(UUIDUtil.getUuid());
            oldNodeIdAndNewMap.put(oldId, item.getId());
            if (StringUtils.isNotBlank(item.getMasterId()) && StringUtils.isBlank(oldNodeIdAndNewMap.get(item.getMasterId()))) {
                oldNodeIdAndNewMap.put(item.getMasterId(), UUIDUtil.getUuid());
            }
            insertNodes.add(item);
        });
        insertNodes.forEach(node -> {
            node.setFatherId(oldNodeIdAndNewMap.get(node.getFatherId()));
            if (FtaNodeEnum.重复事件.getIndex().equals(node.getTypeCode())) {
                node.setMasterId(oldNodeIdAndNewMap.get(node.getMasterId()));
            }
        });
        nodeService.saveBatch(insertNodes);
    }

    /**
     * 根据projectId查出所有分析
     *
     * @param projectId
     * @param userName
     * @return
     */
    List<FtaAnalysis> getAllAnalysisByProjectId(String projectId, String userName) {
        return analysisMapper.selectList(
                new LambdaQueryWrapper<FtaAnalysis>()
                        .select(FtaAnalysis::getAnalysisId, FtaAnalysis::getFatherId, FtaAnalysis::getLevel)
                        .eq(FtaAnalysis::getProjectId, projectId)
                        .eq(FtaAnalysis::getCreateBy, userName));
    }

    /**
     * 清除历史计算数据
     *
     * @param analysis
     * @param allAnalysis
     */
    public void clearCalculateHistory(FtaAnalysis analysis, List<FtaAnalysis> allAnalysis) {
        // 找到该分析项关联的父子分析
        List<String> delAnalysisIds = new ArrayList<>();
        FtaAnalysisService.getAnalysisIds(analysis.getAnalysisId(), delAnalysisIds, allAnalysis);
        // 清除相关节点的计算结果
        nodeMapper.update(FtaNode.builder().q(StringUtils.EMPTY).cutset(StringUtils.EMPTY).build(),
                new LambdaQueryWrapper<FtaNode>().in(FtaNode::getAnalysisId, delAnalysisIds));
    }

    /**
     * @param projectId
     * @param file
     * @param userInfo
     * @return
     */
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_UNCOMMITTED)
    public ServerResponse<String> importAnalysis(String projectId, MultipartFile file, HeaderUserInfo userInfo) {
        long l = System.currentTimeMillis();
        String analysisId = null;
        HashMap<String, String> oldAndNewAnalysisId = new HashMap<>();
        ArrayList<FtaAnalysis> insertAnalysis = new ArrayList<>();
        ArrayList<FtaNode> updateNodes = new ArrayList<>();
        HashMap<String, String> fatherNodeIdAndChildrenId = new HashMap<>();
        HashMap<String, String> oldMiddleIdAndNewId = new HashMap<>();
        // 记录中间事件节点和顺序
        HashMap<String, Integer> middleIdAndOrder = new HashMap<>();
        try (InputStream is = file.getInputStream();
             BufferedReader br = new BufferedReader(new InputStreamReader(is, StandardCharsets.UTF_8))
        ) {
            String line;
            while ((line = br.readLine()) != null) {
                // 一次读入一行数据，每行数据为一个分析
                if (StringUtils.isNotBlank(line)) {
                    analysisId = UUIDUtil.getUuid();
                    FtaImportBo ftaImportBo = JSON.parseObject(line, FtaImportBo.class);
                    if (ftaImportBo != null && ftaImportBo.getTargetNode() != null) {
                        ArrayList<FtaNode> insertNodes = new ArrayList<>();
                        RdfmFaultNetModel topEvent = ftaImportBo.getTargetNode();
                        FtaAnalysis insert = FtaAnalysis.builder()
                                .analysisName(StringUtils.isNotBlank(topEvent.getFaultName()) ? topEvent.getFaultName() : StringUtils.isNotBlank(topEvent.getCode()) ? topEvent.getCode() : topEvent.getStruName() + "-" + topEvent.getFunctionName() + (new SimpleDateFormat("yyyy-MM-dd").format(new Date())))
                                .projectId(projectId).analysisId(analysisId).fatherId(topEvent.getFatherAnalysisId())
                                .level(topEvent.getAnalysisLevel() != null ? topEvent.getAnalysisLevel() : 1)
                                .createTime(new Date())
                                .createBy(userInfo.getUserName()).build();
                        insertAnalysis.add(insert);
                        if (StringUtils.isNotBlank(topEvent.getAnalysisId())) {
                            oldAndNewAnalysisId.put(topEvent.getAnalysisId(), analysisId);
                        }
                        // 顶事件
                        String topEventId = UUIDUtil.getUuid();
                        String topGateId = UUIDUtil.getUuid();
                        insertNodes.add(FtaNode.builder().analysisId(analysisId).id(topEventId).fatherId("0").typeCode(FtaNodeEnum.顶事件.getIndex()).code(StringUtils.isBlank(topEvent.getCode()) ? FtaConstants.TOP_EVENT_CODE : topEvent.getCode()).name(topEvent.getFaultName()).nodeLevel(topEvent.getLevel()).build());
                        // 第一个门
                        insertNodes.add(FtaNode.builder().id(topGateId).fatherId(topEventId).typeCode(StringUtils.isNotBlank(topEvent.getTypeCode()) && StringUtils.isNotBlank(FtaNodeEnum.getNameByIndex(topEvent.getTypeCode())) ? topEvent.getTypeCode() : FtaNodeEnum.或门.getIndex()).requiredPath(topEvent.getRequiredPath() != null && topEvent.getRequiredPath() > 0 ? topEvent.getRequiredPath() : null).analysisId(analysisId).nodeLevel(topEvent.getLevel()).build());
                        if (!CollectionUtils.isEmpty(ftaImportBo.getSubNode())) {

                            List<RdfmFaultNetModel> subNode = ftaImportBo.getSubNode();
                            // 所有存在子节点的id集合，用于区分底事件和门
                            List<String> parentFaultIds = subNode.stream().map(RdfmFaultNetModel::getParentFaultId).collect(Collectors.toList());
                            // 需要添加的所有节点
                            getImportNodeTree(analysisId, parentFaultIds, subNode, insertNodes, topEvent.getFaultId(), topGateId, oldMiddleIdAndNewId, middleIdAndOrder);
                        }
                        String numberStr = "%04d";
                        final Integer[] number = {2};
                        insertNodes.forEach(node -> {
                            if (StringUtils.isBlank(node.getCode()) && FtaNodeEnum.中间事件.getIndex().equals(node.getTypeCode())) {
                                node.setCode(FtaConstants.EVENT_CODE + String.format(numberStr, number[0]++));
                            }
                            if (FtaNodeEnum.转移门.getIndex().equals(node.getTypeCode())) {
                                updateNodes.add(FtaNode.builder().id(node.getId()).typeCode(node.getTypeCode()).childrenAnalysisId(node.getChildrenAnalysisId()).build());
                            }
                            if (FtaNodeEnum.重复事件.getIndex().equals(node.getTypeCode())) {
                                updateNodes.add(FtaNode.builder().id(node.getId()).typeCode(node.getTypeCode()).masterId(node.getMasterId()).build());
                            }
                            if (FtaNodeEnum.未展开事件.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.底事件.getIndex().equals(node.getTypeCode())) {
                                fatherNodeIdAndChildrenId.put(node.getFatherId(), node.getId());
                            }
                        });
                        nodeService.saveBatch(insertNodes);
//                        edgeService.saveBatch(insertEdges);
                        if (insertNodes.stream().anyMatch(node -> node.getNodeLevel() == null || (FtaNodeEnum.中间事件.getIndex().equals(node.getTypeCode()) ? node.getEdgeOrder() == null : false))) {
                            // 该文件中不包含层级和顺序时，需要重新计算层级和顺序
                            diagramDataService.disposalLevelAneOrder(analysisId, null, null, null, false);
                        }

                    }
                }
            }
            insertAnalysis.forEach(analysis -> analysis.setFatherId(oldAndNewAnalysisId.get(analysis.getFatherId())));
            ArrayList<FtaNode> update = new ArrayList<>();
            // 更新转移门关联的分析id和重复事件关联的对应事件id
            updateNodes.forEach(node -> {
                if (FtaNodeEnum.转移门.getIndex().equals(node.getTypeCode())) {
                    node.setChildrenAnalysisId(oldAndNewAnalysisId.get(node.getChildrenAnalysisId()));
                } else if (FtaNodeEnum.重复事件.getIndex().equals(node.getTypeCode())) {
                    node.setMasterId(fatherNodeIdAndChildrenId.get(oldMiddleIdAndNewId.get(node.getMasterId())));
                }
                if (StringUtils.isNotBlank(node.getChildrenAnalysisId()) || StringUtils.isNotBlank(node.getMasterId())) {
                    update.add(node);
                }
            });
            if (!CollectionUtils.isEmpty(update)) {nodeMapper.updateNodes(update);}
            analysisMapper.inserts(insertAnalysis);
        } catch (Exception e) {
            e.printStackTrace();
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.导入失败, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
        log.error("数据导入耗时：" + (System.currentTimeMillis() - l));
        return ServerResponse.createBySuccess(analysisId);
    }

    /**
     * 通过递归找出父子结构并整理为fta的insert数据
     *
     * @param analysisId
     * @param parentFaultIds
     * @param subNode
     * @param inserts
     * @param rdfmFatherId
     * @param ftaFatherId
     */
    void getImportNodeTree(String analysisId, List<String> parentFaultIds, List<RdfmFaultNetModel> subNode, ArrayList<FtaNode> inserts, String rdfmFatherId, String ftaFatherId, HashMap<String, String> oldMiddleIdAndNewId, HashMap<String, Integer> middleIdAndOrder) {
        if (StringUtils.isBlank(rdfmFatherId) || CollectionUtils.isEmpty(subNode)) {
            return;
        }
        subNode.stream().forEach(node -> {
            if (StringUtils.isNotBlank(node.getParentFaultId()) && rdfmFatherId.equals(node.getParentFaultId())) {
                String eventId = UUIDUtil.getUuid();
                String gateId = UUIDUtil.getUuid();
                // 添加中间事件
                inserts.add(FtaNode.builder().analysisId(analysisId).id(eventId).fatherId(ftaFatherId).typeCode(FtaNodeEnum.中间事件.getIndex()).code(node.getCode()).name(node.getFaultName()).nodeLevel(node.getLevel()).build());
                middleIdAndOrder.put(eventId, node.getOrder());
                middleIdAndOrder.put(gateId, 0);
                oldMiddleIdAndNewId.put(node.getFaultId(), eventId);
                if (parentFaultIds.contains(node.getFaultId())) {
                    // id下还有子id的话，添加为基本门，否则添加为基本事件
                    inserts.add(FtaNode.builder().analysisId(analysisId).id(gateId).fatherId(eventId).typeCode(StringUtils.isNotBlank(FtaNodeEnum.getNameByIndex(node.getTypeCode())) ? node.getTypeCode() : FtaNodeEnum.或门.getIndex()).requiredPath(node.getRequiredPath() != null && node.getRequiredPath() > 0 ? node.getRequiredPath() : null).nodeLevel(node.getLevel()).build());
                } else {
                    // 添加无子集的事件
                    // 如果添加的是底事件 暂时只能设定为FixedProb模型类型并只有一个参数（待完善）
                    inserts.add(FtaNode.builder()
                            .analysisId(analysisId)
                            .id(gateId)
                            .fatherId(eventId)
                            .typeCode(StringUtils.isNotBlank(FtaNodeEnum.getNameByIndex(node.getTypeCode())) ? node.getTypeCode() : FtaNodeEnum.底事件.getIndex())
                            .p1Value(node.getP1Value() != null && node.getP1Value() > 0 ? node.getP1Value() : 0.01)
                            .p2Value(node.getP2Value() != null && node.getP2Value() > 0 ? node.getP2Value() : null)
                            .p3Value(node.getP3Value() != null && node.getP3Value() > 0 ? node.getP3Value() : null)
                            .distributionTypeCode(StringUtils.isNotBlank(FtaDistributingTypeCodeEnum.getEnglishNameByIndex(node.getDistributionTypeCode())) ? node.getDistributionTypeCode() : FtaDistributingTypeCodeEnum.FixedProb.getIndex())
                            .childrenAnalysisId(node.getTransferChildrenAnalysisId())
                            .masterId(node.getMasterFatherId())
                            .nodeLevel(node.getLevel()).build());
                }
                getImportNodeTree(analysisId, parentFaultIds, subNode, inserts, node.getFaultId(), gateId, oldMiddleIdAndNewId, middleIdAndOrder);
            }
        });
    }

    public void exportAnalysis(HttpServletResponse response, String analysisId, HeaderUserInfo userInfo) {
        if (StringUtils.isBlank(analysisId)) {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请检查数据, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
        FtaAnalysis analysis = analysisMapper.selectById(analysisId);
        HashMap<String, Integer> analysisIdAndLevelMap = new HashMap<>();
        List<FtaNode> nodes;
        // 判断该分析是否包含父子分析
        if (analysisMapper.selectCount(
                new LambdaQueryWrapper<FtaAnalysis>().eq(FtaAnalysis::getFatherId, analysisId)) > 0
                || nodeMapper.selectCount(
                new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex())
                        .eq(FtaNode::getChildrenAnalysisId, analysisId)) > 0) {
            // 获取该分析id有关联的所有父子分析id
            List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(analysis.getProjectId(), userInfo.getUserId());
            ArrayList<String> analysisIds = new ArrayList<>();
            FtaAnalysisService.getAnalysisIds(analysisId, analysisIds, allAnalysis);

            allAnalysis.forEach(item -> {
                if (analysisIds.contains(item.getAnalysisId())) {
                    analysisIdAndLevelMap.put(item.getAnalysisId(), item.getLevel());
                }
            });
            nodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                    .in(FtaNode::getAnalysisId, analysisIds));
        } else {
            nodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                    .eq(FtaNode::getAnalysisId, analysisId));
            analysisIdAndLevelMap.put(analysis.getAnalysisId(), analysis.getLevel());
        }
        //
        HashMap<String, String> analysisIdAnaFatherId = new HashMap<>();
        HashMap<String, String> transferFatherIdAndChildrenAnalysisId = new HashMap<>();
        nodes.stream().forEach(node -> {
            if (FtaNodeEnum.转移门.getIndex().equals(node.getTypeCode())) {
                analysisIdAnaFatherId.put(node.getChildrenAnalysisId(), node.getAnalysisId());
                transferFatherIdAndChildrenAnalysisId.put(node.getFatherId(), node.getChildrenAnalysisId());
            }
        });
        Map<String, List<FtaNode>> groupByAnalysisId = nodes.stream().collect(Collectors.groupingBy(FtaNode::getAnalysisId));
        StringBuilder result = new StringBuilder();
        // 顶事件和中间事件的id 与 下面的第一个子节点的类型
        HashMap<String, String> middleIdAndChildrenType = new HashMap<>();
        // 表决门的父中间事件id 与 表决门值
        HashMap<String, Integer> knFatherIdAndValue = new HashMap<>();
        // 重复事件的父中间事件id 与 关联的父中间事件id
        HashMap<String, String> mirrorFatherIdAndMasterId = new HashMap<>();
        // 底事件父节点的id 与 基本事件的计算参数
        HashMap<String, Double> nodeFatherIdAndP1Value = new HashMap<>();
        HashMap<String, Double> nodeFatherIdAndP2Value = new HashMap<>();
        HashMap<String, Double> nodeFatherIdAndP3Value = new HashMap<>();
        HashMap<String, String> nodeFatherIdAndDisType = new HashMap<>();
        // 节点的id和fatherid对应关系
        HashMap<String, String> idAndFatherIdMap = new HashMap<>();
        // 节点的id和编码对应关系
        HashMap<String, String> idAndCodeMap = new HashMap<>();
        // 节点的id和这个节点的顺序
        HashMap<String, Integer> idAndOrderMap = new HashMap<>();
        List<String> middleIds = new ArrayList<>();
        // 整理数据
        groupByAnalysisId.forEach((groupAnalysisId, groupNodes) -> {
            groupNodes.stream().forEach(node -> {
                if (FtaNodeEnum.或门.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.与门.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.转移门.getIndex().equals(node.getTypeCode())) {
                    middleIdAndChildrenType.put(node.getFatherId(), node.getTypeCode());
                } else if (FtaNodeEnum.表决门.getIndex().equals(node.getTypeCode())) {
                    middleIdAndChildrenType.put(node.getFatherId(), node.getTypeCode());
                    knFatherIdAndValue.put(node.getFatherId(), node.getRequiredPath());
                } else if (FtaNodeEnum.重复事件.getIndex().equals(node.getTypeCode())) {
                    middleIdAndChildrenType.put(node.getFatherId(), node.getTypeCode());
                    mirrorFatherIdAndMasterId.put(node.getFatherId(), node.getMasterId());
                    nodeFatherIdAndP1Value.put(node.getFatherId(), node.getP1Value());
                    nodeFatherIdAndP2Value.put(node.getFatherId(), node.getP2Value());
                    nodeFatherIdAndP3Value.put(node.getFatherId(), node.getP3Value());
                    nodeFatherIdAndDisType.put(node.getFatherId(), node.getDistributionTypeCode());
                } else if (FtaNodeEnum.底事件.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.未展开事件.getIndex().equals(node.getTypeCode())) {
                    middleIdAndChildrenType.put(node.getFatherId(), node.getTypeCode());
                    nodeFatherIdAndP1Value.put(node.getFatherId(), node.getP1Value());
                    nodeFatherIdAndP2Value.put(node.getFatherId(), node.getP2Value());
                    nodeFatherIdAndP3Value.put(node.getFatherId(), node.getP3Value());
                    nodeFatherIdAndDisType.put(node.getFatherId(), node.getDistributionTypeCode());
                } else if (FtaNodeEnum.中间事件.getIndex().equals(node.getTypeCode())) {
                    middleIds.add(node.getId());
                }
                idAndFatherIdMap.put(node.getId(), node.getFatherId());
                idAndCodeMap.put(node.getId(), node.getCode());
            });
        });
        // 输出
        // 每个分析为一行数据，循环输出多个分析
        groupByAnalysisId.forEach((groupAnalysisId, groupNodes) -> {
            FtaImportBo ftaImportBo = new FtaImportBo();
            ArrayList<RdfmFaultNetModel> subNode = new ArrayList<>();
            groupNodes.stream().forEach(node -> {
                RdfmFaultNetModel model = new RdfmFaultNetModel();
                if (FtaNodeEnum.顶事件.getIndex().equals(node.getTypeCode())) {
                    model.setFaultId(node.getId());
                    model.setFaultName(node.getName());
                    model.setCode(idAndCodeMap.get(node.getId()));
                    model.setTypeCode(middleIdAndChildrenType.get(node.getId()));
                    model.setRequiredPath(knFatherIdAndValue.get(node.getId()));
                    model.setAnalysisLevel(analysisIdAndLevelMap.get(node.getAnalysisId()));
                    model.setAnalysisId(node.getAnalysisId());
                    model.setFatherAnalysisId(analysisIdAnaFatherId.get(node.getAnalysisId()));
                    model.setLevel(node.getNodeLevel());
                    model.setOrder(1);
                    ftaImportBo.setTargetNode(model);
                } else if (FtaNodeEnum.中间事件.getIndex().equals(node.getTypeCode())) {
                    model.setFaultId(node.getId());
                    model.setParentFaultId(idAndFatherIdMap.get(node.getFatherId()));
                    model.setFaultName(node.getName());
                    model.setCode(idAndCodeMap.get(node.getId()));
                    model.setTypeCode(middleIdAndChildrenType.get(node.getId()));
                    model.setRequiredPath(knFatherIdAndValue.get(node.getId()));
                    model.setDistributionTypeCode(nodeFatherIdAndDisType.get(node.getId()));
                    model.setP1Value(nodeFatherIdAndP1Value.get(node.getId()));
                    model.setP2Value(nodeFatherIdAndP2Value.get(node.getId()));
                    model.setP3Value(nodeFatherIdAndP3Value.get(node.getId()));
                    model.setTransferChildrenAnalysisId(transferFatherIdAndChildrenAnalysisId.get(node.getId()));
                    model.setMasterFatherId(idAndFatherIdMap.get(mirrorFatherIdAndMasterId.get(node.getId())));
                    model.setOrder(node.getEdgeOrder());
                    model.setLevel(node.getNodeLevel());
                    subNode.add(model);
                }
            });
            ftaImportBo.setSubNode(subNode);
            result.append(JSON.toJSONString(ftaImportBo) + "\r\n");
        });

        String fileName = new SimpleDateFormat("yyyy-MM-dd").format(new Date()) + ".fta";
        response.setContentType("text/plain");

        try {
            response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        ServletOutputStream outputStream;
        BufferedOutputStream buffer;

        try {
            outputStream = response.getOutputStream();
            buffer = new BufferedOutputStream(outputStream);
            buffer.write(result.toString().getBytes(StandardCharsets.UTF_8));
            buffer.flush();
            buffer.close();
            outputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 节点层级和顺序修复
     *
     * @param analysisId
     * @param userInfo
     * @return
     */
    public ServerResponse<String> repairDiagramData(String analysisId, HeaderUserInfo userInfo) {
        try {
            HashMap<String, Integer> middleIdAndLevelMap = new HashMap<>();
            final String[] topEventId = {null};
            List<FtaNode> allNode = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId));
            List<String> middleNodeIds = allNode.stream().filter(node -> {
                if (FtaNodeEnum.顶事件.getIndex().equals(node.getTypeCode())) {
                    topEventId[0] = node.getId();
                }
                // 原始的level
                middleIdAndLevelMap.put(node.getId(), node.getNodeLevel());
                return FtaNodeEnum.中间事件.getIndex().equals(node.getTypeCode());
            }).map(FtaNode::getId).collect(Collectors.toList());
            List<FtaEdge> allEdge = FtaDiagramDataService.generateEdgesByNodes(allNode);
            List<FtaNode> nodes = new ArrayList<>();
            List<FtaNode> updateNodes = new ArrayList<>();
            List<FtaEdge> updateEdges = new ArrayList<>();
            HashMap<String, Integer> levelMap = new HashMap<>();
            HashMap<String, Integer> orderMap = new HashMap<>();
            // 设定该顶事件的初始层级
            levelMap.put(topEventId[0], 1);
            // 修复连线的顺序
            Map<String, List<FtaEdge>> map = allEdge.stream().collect(Collectors.groupingBy(FtaEdge::getSourceId));

            for (List<FtaEdge> edges : map.values()) {
                List<FtaEdge> targetEdges = edges.stream().sorted(Comparator.comparing(FtaEdge::getEdgeOrder, Comparator.nullsLast(Integer::compareTo))).collect(Collectors.toList());
                if (targetEdges.size() == 1 && (targetEdges.get(0).getEdgeOrder() == null || targetEdges.get(0).getEdgeOrder() != 1)) {
                    updateEdges.add(FtaEdge.builder().targetId(targetEdges.get(0).getTargetId()).edgeOrder(1).build());
                } else if (targetEdges.size() > 1) {
                    // 门下的多条连线
                    targetEdges = targetEdges.stream().sorted(Comparator.comparing(FtaEdge::getEdgeOrder, Comparator.nullsLast(Integer::compareTo))).collect(Collectors.toList());
                    for (int i = 0; i < targetEdges.size(); i++) {
                        if (targetEdges.get(i).getEdgeOrder() == null || targetEdges.get(i).getEdgeOrder() != (i + 1)) {
                            updateEdges.add(FtaEdge.builder().targetId(targetEdges.get(i).getTargetId()).edgeOrder(i + 1).build());
                        }
                    }
                }
            }
            diagramDataService.disposalLevelAneOrder(allEdge, middleNodeIds, topEventId[0], nodes, updateEdges, levelMap, orderMap, true);
            nodes.forEach(node -> {
                if (node.getNodeLevel() != middleIdAndLevelMap.get(node.getId())) {
                    updateNodes.add(node);
                }
            });
            if (!CollectionUtils.isEmpty(updateNodes)) {
                nodeMapper.updateNodes(updateNodes);
            }
            if (!CollectionUtils.isEmpty(updateEdges)) {
                updateNodes.clear();
                updateEdges.forEach(edge -> updateNodes.add(FtaNode.builder().id(edge.getTargetId()).edgeOrder(edge.getEdgeOrder()).build()));
                nodeMapper.updateNodes(updateNodes);
            }
            return ServerResponse.createBySuccess();
        } catch (Exception e) {
            e.printStackTrace();
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.修复失败, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
    }
}
