package com.nationrel.modules.fta.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.nationrel.modules.fta.bo.FtaGateBo;
import com.nationrel.modules.fta.bo.InputFileDataBo;
import com.nationrel.modules.fta.config.AllVariableConfig;
import com.nationrel.modules.fta.constant.FtaConstants;
import com.nationrel.modules.fta.entity.*;
import com.nationrel.modules.fta.general.common.ResponseStatus;
import com.nationrel.modules.fta.general.common.ServerResponse;
import com.nationrel.modules.fta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.fta.general.domain.enums.fta.*;
import com.nationrel.modules.fta.general.domain.enums.weibull.EwbConstants;
import com.nationrel.modules.fta.general.exception.DescribeException;
import com.nationrel.modules.fta.general.util.common.BasicDataUtil;
import com.nationrel.modules.fta.general.util.common.UUIDUtil;
import com.nationrel.modules.fta.mapper.FtaAnalysisMapper;
import com.nationrel.modules.fta.mapper.FtaNodeMapper;
import com.nationrel.modules.fta.mapper.FtaOperatorDetailMapper;
import com.nationrel.modules.fta.mapper.FtaOperatorMapper;
import com.nationrel.modules.fta.util.FileUtil;
import com.nationrel.modules.fta.vo.FtaEventVo;
import com.nationrel.modules.fta.vo.FtaImageVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.io.*;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * fta页面操作
 *
 * @author zhangyuanmin
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FtaDiagramDataService {

    private final AllVariableConfig allVariableConfig;
    private final FtaAnalysisMapper analysisMapper;
    private final FtaNodeMapper nodeMapper;
    private final FtaOperatorMapper operatorMapper;
    private final FtaOperatorService operatorService;
    private final FtaOperatorDetailMapper operatorDetailMapper;
    private final FtaNodeService nodeService;
    private final FtaRgaApiService ftaRgaApiService;

    /**
     * 根据ftaId查询节点和连线，整理数据后返回（补点并排序：用于实现视图顺序功能）
     *
     * @param analysisId
     * @return
     */
    public ServerResponse<List<JSONObject>> getNodesData(String analysisId) {

        List<FtaNode> nodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId));
        List<FtaEdge> edges = generateEdgesByNodes(nodes);

        FtaAnalysis analysis = analysisMapper.selectById(analysisId);
        if (FtaAnalysisViewTypeEnum.紧凑.getIndex().equals(analysis.getViewType())) {
            // 紧凑排列时不排序
            List<JSONObject> result = nodes.stream().map(item -> {
                if (!FtaNodeEnum.中间事件.getIndex().equals(item.getTypeCode())) {
                    item.setDisableMove(null);
                    item.setLeaf(null);
                }
                return JSON.parseObject(JSON.toJSONString(item));
            }).collect(Collectors.toList());
            edges.forEach(item -> result.add(JSON.parseObject(JSON.toJSONString(item))));
            return ServerResponse.createBySuccess(result);
        } else if (FtaAnalysisViewTypeEnum.宽松.getIndex().equals(analysis.getViewType())) {
            // 查询最低层级的数据
            int maxLevel = nodeMapper.selectMaxnodeLevel(analysisId);
            calculateNode(analysisId, maxLevel, nodes, edges);
            return ServerResponse.createBySuccess(comparingNodeAndEdge(nodes, edges));
        } else {
            // 默认为常规（仅补少量候补节点）
            // 查询最低层级的数据
            int maxLevel = nodeMapper.selectMaxnodeLevel(analysisId);
            if (maxLevel <= 4) {
                calculateNode(analysisId, maxLevel, nodes, edges);
            } else {
                // 大于4个层级补固定结构
                String sourceNodeId = nodeMapper.selectFirstLevelNodeId(analysisId);
                addLeftAndRight(nodes, edges, analysisId, sourceNodeId);
            }
            return ServerResponse.createBySuccess(comparingNodeAndEdge(nodes, edges));
        }
    }

    public static List<FtaEdge> generateEdgesByNodes(List<FtaNode> nodes) {
        ArrayList<FtaEdge> edges = new ArrayList<>();
        nodes.forEach(node -> {
            if (!FtaNodeEnum.顶事件.getIndex().equals(node.getTypeCode())) {
                edges.add(FtaEdge.builder().sourceId(node.getFatherId()).targetId(node.getId()).edgeOrder(node.getEdgeOrder()).build());
            }
        });
        return edges;
    }

    /**
     * 将连线和节点排序
     *
     * @param nodes
     * @param edges
     * @return
     */
    private List<JSONObject> comparingNodeAndEdge(List<FtaNode> nodes, List<FtaEdge> edges) {
        // 排序连线
        ArrayList<JSONObject> orderNodes = new ArrayList<>();
        nodes.forEach(item -> orderNodes.add(JSON.parseObject(JSON.toJSONString(item))));
        edges.forEach(item -> orderNodes.add(JSON.parseObject(JSON.toJSONString(item))));
        List<JSONObject> result = orderNodes.stream().sorted(Comparator.comparing(js -> js.getInteger("edgeOrder"), Comparator.nullsLast(Integer::compareTo))).collect(Collectors.toList());
        return result;
    }

    /**
     * 补节点
     */
    private void calculateNode(String analysisId, int maxLevel, List<FtaNode> nodes, List<FtaEdge> edges) {
        List<FtaNode> lastLevel = nodeMapper.selectLastLevel(analysisId, maxLevel);
        // 补层级
        if (!CollectionUtils.isEmpty(lastLevel)) {
            for (FtaNode node : lastLevel) {
                String sourceNodeId = node.getId();
                for (int i = node.getNodeLevel(); i < maxLevel; i++) {
                    String targetNodeId = UUIDUtil.getUuid();
                    addNodeEdge(nodes, edges, analysisId, sourceNodeId, targetNodeId, 0);
                    sourceNodeId = targetNodeId;
                    targetNodeId = UUIDUtil.getUuid();
                    addNodeEdge(nodes, edges, analysisId, sourceNodeId, targetNodeId, 0);
                    sourceNodeId = targetNodeId;
                }
            }
        }
    }

    /**
     * 补最左和最右结构(for (int i = 1; i < 2; i++) i=0补最左加最右，i=1补最右)
     *
     * @param nodes
     * @param edges
     * @param analysisId
     * @param sourceId
     */
    private void addLeftAndRight(List<FtaNode> nodes, List<FtaEdge> edges, String analysisId, String sourceId) {
        int maxOrder = nodeMapper.selectMaxTwoLevelOrder(analysisId, sourceId);
        for (int i = 1; i < 2; i++) {
            String targetNodeId = UUIDUtil.getUuid();
            addNodeEdge(nodes, edges, analysisId, sourceId, targetNodeId, maxOrder + 1);
            String sourceNodeId = targetNodeId;
            targetNodeId = UUIDUtil.getUuid();
            addNodeEdge(nodes, edges, analysisId, sourceNodeId, targetNodeId, 0);
            addNodeEdge(nodes, edges, analysisId, sourceNodeId, UUIDUtil.getUuid(), 1);
            addNodeEdge(nodes, edges, analysisId, targetNodeId, UUIDUtil.getUuid(), 0);
        }
    }

    private void addNodeEdge(List<FtaNode> nodes, List<FtaEdge> edges, String analysisId, String sourceNodeId, String targetNodeId, int order) {
        edges.add(FtaEdge.builder().id(UUIDUtil.getUuid()).analysisId(analysisId).sourceId(sourceNodeId).targetId(targetNodeId).edgeOrder(order).build());
        nodes.add(FtaNode.builder().id(UUIDUtil.getUuid()).analysisId(analysisId).id(targetNodeId).fatherId(sourceNodeId).typeCode(FtaNodeEnum.候补节点.getIndex()).build());
    }

    /**
     * 添加节点：添加中间事件或其它基本事件
     *
     * @param entity
     * @param userInfo
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> insert(FtaEventVo entity, HeaderUserInfo userInfo) {
        Long edgeOrder = nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getFatherId, entity.getSourceId()));
        FtaNode fatherNode = nodeMapper.selectById(entity.getSourceId());
        String analysisId = fatherNode.getAnalysisId();
        // 清除计算历史数据
        String projectId = analysisMapper.selectOne(new LambdaQueryWrapper<FtaAnalysis>()
                .select(FtaAnalysis::getProjectId).eq(FtaAnalysis::getAnalysisId, analysisId)).getProjectId();
        clearCalculateHistory(userInfo, analysisId, projectId);
        String eventId = UUIDUtil.getUuid();
        String childId = UUIDUtil.getUuid();
        // 记录新增的id
        ArrayList<String> operatorInsertNodeIds = new ArrayList<>();
        ArrayList<String> operatorInsertEdgeIds = new ArrayList<>();
        if (FtaNodeEnum.底事件.getIndex().equals(entity.getTypeCode())
                || FtaNodeEnum.重复事件.getIndex().equals(entity.getTypeCode())
                || FtaNodeEnum.未展开事件.getIndex().equals(entity.getTypeCode())) {
            FtaNode masterNode = null;
            if (FtaNodeEnum.重复事件.getIndex().equals(entity.getTypeCode())) {
                // 添加重复事件时，同一个父中间事件下不能存在相同的重复事件（也包括被重复的底事件）
                List<String> ids = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                                .select(FtaNode::getId)
                                .eq(FtaNode::getAnalysisId, analysisId)
                                .eq(FtaNode::getFatherId, fatherNode.getId()))
                        .stream().map(FtaNode::getId).collect(Collectors.toList());
                Long count = nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>()
                        .eq(FtaNode::getAnalysisId, analysisId)
                        .in(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex(), FtaNodeEnum.底事件.getIndex())
                        .in(FtaNode::getFatherId, ids)
                        .and(o -> o.eq(FtaNode::getMasterId, entity.getMasterId()).or().eq(FtaNode::getId, entity.getMasterId())));
                if (!CollectionUtils.isEmpty(ids) && count > 0) {
                    throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.同一个门下禁止添加同一个重复事件, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
                }
                // 不再区分底事件和重复事件，都变为重复事件，存储一个相同的masterId, 存储相同的数据，都一起更新
                masterNode = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                        .in(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex(), FtaNodeEnum.底事件.getIndex())
                        .and(o -> o.eq(FtaNode::getMasterId, entity.getMasterId()).or()
                                .eq(FtaNode::getId, entity.getMasterId()))
                        .isNotNull(FtaNode::getDistributionTypeCode)
                        .isNotNull(FtaNode::getP1Value)
                        .last(" LIMIT 1"));
                if (nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>()
                        .in(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex(), FtaNodeEnum.底事件.getIndex())
                        .and(o -> o.eq(FtaNode::getMasterId, entity.getMasterId()).or()
                                .eq(FtaNode::getId, entity.getMasterId()))) == 1) {
                    // 如果是第一次添加重复事件，则需要将原来的底事件也转为重复事件
                    nodeMapper.updateById(FtaNode.builder().id(masterNode.getId())
                            .masterId(StringUtils.isNotBlank(masterNode.getMasterId()) ? masterNode.getMasterId() : masterNode.getId())
                            .typeCode(FtaNodeEnum.重复事件.getIndex()).build());
                }
            }
            // 添加其他事件时，实际添加的是中间事件和一个事件
            // 添加中间事件
            FtaNode middleEvent = FtaNode.builder()
                    .analysisId(entity.getAnalysisId())
                    .id(eventId)
                    .fatherId(entity.getSourceId())
                    .nodeLevel(fatherNode.getNodeLevel() + 1)
                    .edgeOrder((int) (edgeOrder + 1))
                    .name(entity.getName()).code(entity.getCode())
                    .typeCode(FtaNodeEnum.中间事件.getIndex()).build();
            nodeMapper.insert(middleEvent);
            // 添加底事件或重复事件 或未展开事件
            FtaNode event = FtaNode.builder()
                    .analysisId(entity.getAnalysisId())
                    .id(childId)
                    .fatherId(middleEvent.getId())
                    .nodeLevel(fatherNode.getNodeLevel() + 1)
                    .masterId(entity.getMasterId())
                    .typeCode(entity.getTypeCode())
                    .distributionTypeCode(entity.getDistributionTypeCode())
                    .p1Value(entity.getP1Value())
                    .build();
            if (FtaNodeEnum.重复事件.getIndex().equals(entity.getTypeCode()) && masterNode != null) {
                // 如果添加的是重复事件，则新添加的计算参数设置为原master的数据
                event.setMasterId(StringUtils.isNotBlank(masterNode.getMasterId()) ? masterNode.getMasterId() : masterNode.getId());
                event.setDistributionTypeCode(masterNode.getDistributionTypeCode());
                event.setP1Value(masterNode.getP1Value());
                event.setP2Value(masterNode.getP2Value());
                event.setP3Value(masterNode.getP3Value());
            }
            nodeMapper.insert(event);

            operatorInsertNodeIds.add(middleEvent.getId());
            operatorInsertNodeIds.add(event.getId());
            operatorService.insert(userInfo, operatorInsertNodeIds, operatorInsertEdgeIds, analysisId, null, FtaOperatorTypeEnum.新增节点和连线.getIndex());
            return ServerResponse.createBySuccess();
        } else if (FtaNodeEnum.中间事件.getIndex().equals(entity.getTypeCode())) {
            if (StringUtils.isBlank(entity.getCode())) {
                entity.setCode(FtaConstants.EVENT_CODE + getEventCode(fatherNode.getAnalysisId(), userInfo).getData());
            }
            // 添加中间事件时，实际添加的是中间事件和一个门
            FtaNode middleEvent = FtaNode.builder()
                    .analysisId(entity.getAnalysisId())
                    .id(eventId)
                    .typeCode(entity.getTypeCode())
                    .code(entity.getCode())
                    .name(entity.getName())
                    .fatherId(entity.getSourceId())
                    .nodeLevel(fatherNode.getNodeLevel() + 1)
                    .edgeOrder((int) (edgeOrder + 1))
                    .build();
            nodeMapper.insert(middleEvent);

            FtaNode gate = FtaNode.builder()
                    .analysisId(entity.getAnalysisId())
                    .fatherId(middleEvent.getId())
                    .nodeLevel(fatherNode.getNodeLevel() + 1)
                    .id(childId)
                    .childrenAnalysisId(StringUtils.isNotBlank(entity.getChildrenTypeCode()) && FtaNodeEnum.转移门.getIndex().equals(entity.getChildrenTypeCode()) ? UUIDUtil.getUuid() : null)
                    .requiredPath(StringUtils.isNotBlank(entity.getChildrenTypeCode()) && FtaNodeEnum.表决门.getIndex().equals(entity.getChildrenTypeCode()) ? 2 : null)
                    .typeCode(StringUtils.isNotBlank(entity.getChildrenTypeCode()) && !FtaNodeEnum.顶事件.getIndex().equals(entity.getChildrenTypeCode()) && !FtaNodeEnum.中间事件.getIndex().equals(entity.getChildrenTypeCode()) ? entity.getChildrenTypeCode() : FtaNodeEnum.或门.getIndex()).build();
            nodeMapper.insert(gate);

            String result = StringUtils.EMPTY;
            if (StringUtils.isNotBlank(gate.getChildrenAnalysisId())) {
                // 当新添加的是一个中间事件和转移门时
                FtaAnalysis fatherAnalysis = analysisMapper.selectById(fatherNode.getAnalysisId());
                // 为新的转移门添加一个子分析
                FtaAnalysis insert = FtaAnalysis.builder()
                        .projectId(fatherAnalysis.getProjectId())
                        .analysisId(gate.getChildrenAnalysisId())
                        .fatherId(fatherAnalysis.getAnalysisId())
                        .level(fatherAnalysis.getLevel() + 1)
                        .analysisName(middleEvent.getCode())
                        .createTime(new Date())
                        .createBy(userInfo.getUserName())
                        .updateTime(new Date())
                        .updateBy(userInfo.getUserName())
                        .delFlag(FtaConstants.DEFAULT_DEL_FLAG).build();
                analysisMapper.insert(insert);
                initDiagramData(gate.getChildrenAnalysisId(), middleEvent.getName(), middleEvent.getCode());
                result = gate.getChildrenAnalysisId();
            }

            operatorInsertNodeIds.add(middleEvent.getId());
            operatorInsertNodeIds.add(gate.getId());
            ArrayList<String> newAnalysisIds = new ArrayList<>();
            if (StringUtils.isNotBlank(gate.getChildrenAnalysisId())) {
                newAnalysisIds.add(gate.getChildrenAnalysisId());
            }
            operatorService.insert(userInfo, operatorInsertNodeIds, operatorInsertEdgeIds, analysisId, newAnalysisIds, FtaOperatorTypeEnum.新增节点和连线.getIndex());

            return ServerResponse.createBySuccess(result);
        }
        return ServerResponse.createByError();
    }

    /**
     * 清除历史计算数据（如果该分析存在关联的父子分析，同时清除这些分析的计算历史数据）
     *
     * @param ftaId
     */
    public void clearCalculateHistory(HeaderUserInfo userInfo, String ftaId, String projectId) {
        // 找到该分析项关联的父子分析
        List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(projectId, userInfo.getUserId());
        List<String> delAnalysisIds = new ArrayList<>();
        FtaAnalysisService.getAnalysisIds(ftaId, delAnalysisIds, allAnalysis);
        // 清除相关节点的计算结果
        delAnalysisIds.forEach(analysisId -> nodeMapper.update(FtaNode.builder().build(),
                new LambdaUpdateWrapper<FtaNode>().set(FtaNode::getQ, null).set(FtaNode::getCutset, null)
                        .eq(FtaNode::getAnalysisId, analysisId)));
    }

    /**
     * 初始化故障树数据：添加一个顶事件和一个门并连线（该方法用于创建分析时）
     */
    @Transactional(rollbackFor = Exception.class)
    public void initDiagramData(String analysisId, String analysisName, String code) {

        String topEventId = UUIDUtil.getUuid();
        FtaNode topEvent = FtaNode.builder()
                .analysisId(analysisId)
                .code(StringUtils.isNotBlank(code) ? code : FtaConstants.TOP_EVENT_CODE)
                .name(analysisName)
                .fatherId("0")
                .nodeLevel(1)
                .edgeOrder(1)
                .id(topEventId)
                .typeCode(FtaNodeEnum.顶事件.getIndex()).build();
        nodeMapper.insert(topEvent);

        String andGateId = UUIDUtil.getUuid();
        FtaNode andGate = FtaNode.builder()
                .analysisId(analysisId)
                .fatherId(topEvent.getId())
                .nodeLevel(1)
                .id(andGateId)
                .typeCode(FtaNodeEnum.与门.getIndex()).build();
        nodeMapper.insert(andGate);

    }

    /**
     * 更新节点（或节点类型转换）
     *
     * @param entity 实际被操作的门或者底事件
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse update(HeaderUserInfo userInfo, FtaNode entity, boolean operator) {
        FtaNode old = nodeMapper.selectById(entity.getId());
        FtaNode fatherNode = nodeMapper.selectById(old.getFatherId());
        String analysisId = old.getAnalysisId();
        String projectId = analysisMapper.selectOne(new LambdaQueryWrapper<FtaAnalysis>()
                .select(FtaAnalysis::getProjectId).eq(FtaAnalysis::getAnalysisId, analysisId)).getProjectId();
        JSONObject result = new JSONObject();
        if (!FtaNodeEnum.转移门.getIndex().equals(old.getTypeCode())
                && FtaNodeEnum.转移门.getIndex().equals(entity.getTypeCode())) {
            // 将一个基本门转为转移门
            FtaNode gate = nodeMapper.selectById(entity.getId());
            FtaNode middle = nodeMapper.selectById(gate.getFatherId());
            return toTransferGate(userInfo, gate, middle, operator);
        } else if (FtaNodeEnum.转移门.getIndex().equals(old.getTypeCode())
                && !FtaNodeEnum.转移门.getIndex().equals(entity.getTypeCode())) {
            // 将一个转移门转为子集
            return transferGateToChildren(userInfo, old, fatherNode, projectId, analysisId, operator);
        } else if (FtaNodeEnum.底事件.getIndex().equals(entity.getTypeCode()) &&
                (FtaNodeEnum.与门.getIndex().equals(old.getTypeCode()) || FtaNodeEnum.或门.getIndex().equals(old.getTypeCode()) || FtaNodeEnum.表决门.getIndex().equals(old.getTypeCode()))) {
            // 基本门转底事件
            // 更新节点类型
            FtaNode update = FtaNode.builder()
                    .id(entity.getId()).typeCode(entity.getTypeCode())
                    .distributionTypeCode(FtaDistributingTypeCodeEnum.FixedProb.getIndex())
                    .p1Value(0.01).build();
            nodeMapper.updateById(update);
            // 获取该fta分析的所有连线
            List<FtaNode> nodes = nodeMapper.selectList(new LambdaUpdateWrapper<FtaNode>().eq(FtaNode::getAnalysisId, old.getAnalysisId()));
            List<FtaEdge> edges = generateEdgesByNodes(nodes);
            // 找出需要删除的所有节点id
            ArrayList<String> delNodeIds = new ArrayList<>();
            ArrayList<String> delEdgeIds = new ArrayList<>();
            getRelevanceIdByNodeId(edges, entity.getId(), delNodeIds);
            delNodeIds.remove(entity.getId());
            // 找出需要删除的线
            getEdgeIdsByNodeIds(delEdgeIds, edges, delNodeIds);
            if (CollectionUtils.isEmpty(delNodeIds)) {
                // 当该门下没有子集，则仅记录节点更新操作即可
                operatorService.updateNodeTypeCode(userInfo, old, analysisId, null);
            } else {
                // 删除该门下的数据（不包含该门），需要考虑转移门和额外更新的重复事件及连线，并记录操作用于回退
                deleteGate(userInfo, result, delNodeIds, delEdgeIds, projectId, old, fatherNode, false);
            }
        } else if (StringUtils.isNotBlank(entity.getTypeCode()) && !old.getTypeCode().equals(entity.getTypeCode())) {
            // 简单转换（底事件转为基本门 或 底事件与未展开事件互转）
            // 这些操作仅需更新节点类型字段和该节点的少量参数
            FtaNode update = FtaNode.builder()
                    .id(entity.getId()).name(entity.getName())
                    .code(entity.getCode()).typeCode(entity.getTypeCode())
                    .requiredPath(entity.getRequiredPath()).disableMove(entity.getDisableMove())
                    .leaf(entity.getLeaf()).build();
            nodeMapper.updateById(update);
            // 记录简单转换操作
            operatorService.updateNodeTypeCode(userInfo, old, analysisId, null);
        } else {
            // 更新节点参数（目前不记录更新节点参数的回退操作）
            FtaNode update = FtaNode.builder()
                    .id(entity.getId()).name(entity.getName()).code(entity.getCode())
                    .requiredPath(entity.getRequiredPath())
                    .disableMove(entity.getDisableMove()).leaf(entity.getLeaf()).build();
            if (FtaNodeEnum.底事件.getIndex().equals(entity.getTypeCode())
                    || FtaNodeEnum.未展开事件.getIndex().equals(entity.getTypeCode())
                    || FtaNodeEnum.重复事件.getIndex().equals(entity.getTypeCode())) {
                // 底事件和未展开事件和重复事件：更新计算参数
                update.setDistributionTypeCode(entity.getDistributionTypeCode());
                update.setP1Value(entity.getP1Value());
                update.setP2Value(entity.getP2Value());
                update.setP3Value(entity.getP3Value());
            }
            if (FtaNodeEnum.重复事件.getIndex().equals(entity.getTypeCode())) {
                ArrayList<FtaNode> updateMirrors = new ArrayList<>();
                updateMirrors.add(entity);
                nodeMapper.updateMirrorNodes(updateMirrors);
            } else if (FtaNodeEnum.中间事件.getIndex().equals(old.getTypeCode())) {
                // 更新中间事件的名称和编码时，需要验证该中间事件下面的是否为重复事件
                // 如果是重复事件则需要更新所有关联的重复事件的父节点
                FtaNode child = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                        .eq(FtaNode::getFatherId, old.getId()));
                if (FtaNodeEnum.重复事件.getIndex().equals(child.getTypeCode())) {
                    ArrayList<FtaNode> updateMirrors = new ArrayList<>();
                    updateMirrors.add(FtaNode.builder().code(entity.getCode()).name(entity.getName()).masterId(child.getMasterId()).build());
                    nodeMapper.updateMirrorFatherNodes(updateMirrors);
                } else {
                    nodeMapper.updateById(update);
                }
            } else {
                nodeMapper.updateById(update);
            }
        }
        if (operator && StringUtils.isNotBlank(entity.getDistributionTypeCode())
                || null != entity.getP1Value()
                || StringUtils.isNotBlank(entity.getTypeCode())
                || entity.getMasterId() != null) {
            //清除历史计算数据
            clearCalculateHistory(userInfo, analysisId, projectId);
        }
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 故障树中将子树转为转移门
     *
     * @param userInfo
     * @param gate     被转移的门
     * @param middle   该门的父中间事件
     * @param operator 是否需要记录操作
     * @return
     */
    public ServerResponse<String> toTransferGate(HeaderUserInfo userInfo, FtaNode gate, FtaNode middle, boolean operator) {
        // 将该门下的数据全部拷贝到一个新的分析中，并将原分析设为新分析的父级
        // 创建一个新的子分析（子树）
        FtaAnalysis analysis = new FtaAnalysis();
        String newAnalysisId = BasicDataUtil.uuidGenerator();
        FtaAnalysis fatherAnalysis = analysisMapper.selectById(gate.getAnalysisId());
        analysis.setAnalysisName(middle.getCode());
        analysis.setProjectId(fatherAnalysis.getProjectId());
        analysis.setFatherId(fatherAnalysis.getAnalysisId());
        analysis.setLevel(fatherAnalysis.getLevel() + 1);
        analysis.setCreateTime(new Date());
        analysis.setCreateBy(userInfo.getUserName());
        analysis.setAnalysisId(newAnalysisId);
        analysisMapper.insert(analysis);
        ArrayList<String> updateNodeIds = new ArrayList<>();
        List<FtaNode> nodes = nodeMapper.selectList(
                new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getAnalysisId, gate.getAnalysisId()));
        List<FtaEdge> edges = generateEdgesByNodes(nodes);
        // 将原来的门和门的父级中间事件及连线复制一份，到新的分析中
        FtaNode newGate = new FtaNode();
        FtaNode newMiddle = new FtaNode();
        BeanUtils.copyProperties(gate, newGate);
        BeanUtils.copyProperties(middle, newMiddle);
        newMiddle.setId(UUIDUtil.getUuid());
        newMiddle.setFatherId("0");
        newMiddle.setAnalysisId(newAnalysisId);
        newMiddle.setTypeCode((FtaNodeEnum.顶事件.getIndex()));
        nodeMapper.insert(newMiddle);
        newGate.setId(UUIDUtil.getUuid());
        newGate.setFatherId(newMiddle.getId());
        newGate.setAnalysisId(newAnalysisId);
        nodeMapper.insert(newGate);
        // 将门下的数据的分析id直接更新为新分析id，并更新相关的fatherId
        // 找到需要更新的节点id
        getRelevanceIdByNodeId(edges, gate.getId(), updateNodeIds);
        updateNodeIds.remove(gate.getId());
        if (!CollectionUtils.isEmpty(updateNodeIds)) {
            // 如果该门的子集中已经包含了转移门，则调整这些转移门关联的父分析id及所有子集分析的id
            List<FtaAnalysis> allAnalysis = analysisMapper.selectList(
                    new LambdaQueryWrapper<FtaAnalysis>()
                            .eq(FtaAnalysis::getProjectId, analysis.getProjectId())
                            .eq(FtaAnalysis::getCreateBy, userInfo.getUserName()));
            ArrayList<String> updateAnalysisIds = new ArrayList<>();
            ArrayList<String> updatesLevelByAnalysisId = new ArrayList<>();
            nodes.forEach(node -> {
                // 该门为转移门且该门存在于被转移的子集中
                if (FtaNodeEnum.转移门.getIndex().equals(node.getTypeCode())
                        && updateNodeIds.contains(node.getId())) {
                    allAnalysis.forEach(item -> {
                        if (item.getAnalysisId().equals(node.getChildrenAnalysisId())) {
                            // 需要更新层级的分析id
                            updateAnalysisIds.add(item.getAnalysisId());
                        }
                    });
                }
            });
            // 更新子集中包含的转移门关联的分析的fatherId
            if (!CollectionUtils.isEmpty(updateAnalysisIds)) {
                analysisMapper.update(
                        FtaAnalysis.builder().fatherId(newAnalysisId).build(),
                        new LambdaQueryWrapper<FtaAnalysis>().in(FtaAnalysis::getAnalysisId, updateAnalysisIds));
                updateAnalysisIds.forEach(analysisId -> {
                    // 将这些分析及更深层子分析的层级+1
                    updatesLevelByAnalysisId.add(analysisId);
                    FtaAnalysisService.getChildrenAnalysisIds(analysisId, updatesLevelByAnalysisId, allAnalysis);
                });
                // 更新层级
                analysisMapper.addLevel(updatesLevelByAnalysisId);
            }
            // 更新门下直接连接节点的fatherId和节点类型置为顶事件
            nodeMapper.update(FtaNode.builder().fatherId(newGate.getId()).build(),
                    new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getFatherId, gate.getId()));
            nodeMapper.updateNodesAnalysisId(updateNodeIds, newAnalysisId);
            // 重新整理该分析id下的所有节点和连线的顺序和层级
            disposalLevelAneOrder(newAnalysisId, null, null, null, true);
        }
        // 将原来的门转换为转移门
        gate.setTypeCode(FtaNodeEnum.转移门.getIndex());
        gate.setChildrenAnalysisId(newAnalysisId);
        nodeMapper.updateById(gate);
        if (operator) {
            operatorService.toTransferGate(gate.getId(), gate.getAnalysisId(), userInfo);
        }
        return ServerResponse.createBySuccess(newAnalysisId);
    }

    /**
     * 将子集转为转移门
     *
     * @param userInfo
     * @param old
     * @param middleNode
     * @param projectId
     * @param analysisId
     * @param operator
     * @return
     */
    public ServerResponse<String> transferGateToChildren(HeaderUserInfo userInfo, FtaNode old, FtaNode middleNode, String projectId, String analysisId, boolean operator) {
        // 将一个转移门转为子集
        // 获取该转移门关联的 需要更新的分析id及子树id
        String childrenAnalysisId = old.getChildrenAnalysisId();
        // 删除需要删除的子分析id
        analysisMapper.delete(new LambdaQueryWrapper<FtaAnalysis>()
                .eq(FtaAnalysis::getAnalysisId, childrenAnalysisId));
        // 如果该子分析中包含转移门，则需要将这些分析的层级-1
        List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(projectId, userInfo.getUserId());
        ArrayList<String> needUpdateAnalysisIds = new ArrayList<>();
        FtaAnalysisService.getChildrenAnalysisIds(childrenAnalysisId, needUpdateAnalysisIds, allAnalysis);
        if (!CollectionUtils.isEmpty(needUpdateAnalysisIds)) {
            analysisMapper.subLevel(needUpdateAnalysisIds);
            // 并更新这些分析的父分析为当前分析
            analysisMapper.update(FtaAnalysis.builder().fatherId(analysisId).build(),
                    new LambdaQueryWrapper<FtaAnalysis>().eq(FtaAnalysis::getFatherId, childrenAnalysisId));
        }
        // 获取待更新分析的所有节点和连线
        List<FtaNode> nodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                .in(FtaNode::getAnalysisId, childrenAnalysisId));
        List<FtaEdge> edges = generateEdgesByNodes(nodes);

        // 获取需要删除的多余的节点和连线（当前转移门关联的分析的顶事件和下面第一个门）
        ArrayList<String> delNodeIds = new ArrayList<>();
        ArrayList<String> delEdgeIds = new ArrayList<>();

        final FtaNode[] childTopEvent = {null};
        final FtaNode[] childTopGate = {null};
        // 节点id和子节点idmap
        nodes.forEach(node -> {
            if (FtaNodeEnum.顶事件.getIndex().equals(node.getTypeCode())) {
                // 找到子分析的顶事件
                childTopEvent[0] = node;
            }
        });
        nodes.forEach(node -> {
            if (childTopGate[0] == null && childTopEvent[0].getId().equals(node.getFatherId())) {
                // 找到子分析的第一个门
                childTopGate[0] = node;
            }
        });
        // 仅删除子分析中的顶事件和第一个门及这两个节点之间的连线，不删除第一个门下的连线
        delNodeIds.add(childTopEvent[0].getId());
        getEdgeIdsByNodeIds(delEdgeIds, edges, delNodeIds);
        delNodeIds.add(childTopGate[0].getId());
        // 删除子分析的顶事件和第一个门，及连线
        nodeMapper.deleteBatchIds(delNodeIds);
        // 将子分析的第一个门的部分参数，更新到被操作的那个转移门上
        nodeMapper.updateById(FtaNode.builder()
                .id(old.getId()).childrenAnalysisId(StringUtils.EMPTY)
                .typeCode(childTopGate[0].getTypeCode())
                .requiredPath(childTopGate[0].getRequiredPath()).build());
        // 将该转移门的子集们中剩余的数据整理到此分析下
        ArrayList<String> updateNodeIds = new ArrayList<>();
        ArrayList<String> updateEdgeIds = new ArrayList<>();
        nodes.forEach(node -> {
            if (!node.getId().equals(childTopEvent[0].getId()) && !node.getId().equals(childTopGate[0].getId())) {
                // 更新这些节点的分析id为当前分析的id
                updateNodeIds.add(node.getId());
            }
        });
        // 第一个门下的那些中间事件，更新他们的父id为之前的转移门的id
        nodeMapper.update(FtaNode.builder().fatherId(old.getId()).build(),
                new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getFatherId, childTopGate[0].getId()));
        // 更新相关的节点和连线的分析id
        edges.forEach(edge -> {
            if (!edge.getTargetId().equals(childTopGate[0].getId())) {
                updateEdgeIds.add(edge.getId());
            }
        });
        if (!CollectionUtils.isEmpty(updateNodeIds)) {
            nodeMapper.updateNodesAnalysisId(updateNodeIds, analysisId);
        }
        disposalLevelAneOrder(analysisId, old.getFatherId(), old.getNodeLevel(), middleNode.getEdgeOrder(), true);
        if (operator) {
            operatorService.toSubset(old.getId(), old.getAnalysisId(), userInfo);
        }
        return ServerResponse.createBySuccess();
    }

    /**
     * 重新排列该分析下的所有节点和连线的层级和顺序
     *
     * @param analysisId 需要重新计算层级和顺序的分析id
     * @param fatherId   为空则计算该分析中全部节点和连线的层级和顺序，不为空则仅计算该fatherId下的子集
     * @param level      该fahterId的层级
     * @param order      该fatherId的顺序
     * @param onlyLevel  是否只排列层级，不考虑顺序（该参数仅对于子集生效。被操作的那个顶事件或中间事件一定要重新计算顺序）
     */
    public void disposalLevelAneOrder(String analysisId, String fatherId, Integer level, Integer order, boolean onlyLevel) {
        List<String> middleNodeIds = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId).eq(FtaNode::getTypeCode, FtaNodeEnum.中间事件.getIndex())).stream().map(FtaNode::getId).collect(Collectors.toList());
        // 获取该fta分析的所有连线和节点
        List<FtaNode> nodes = nodeMapper.selectList(new LambdaUpdateWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId));
        List<FtaEdge> allEdge = generateEdgesByNodes(nodes);
        List<FtaNode> updateNodes = new ArrayList<>();
        List<FtaEdge> updateEdges = new ArrayList<>();
        HashMap<String, Integer> levelMap = new HashMap<>();
        HashMap<String, Integer> orderMap = new HashMap<>();
        FtaNode topEvent;
        if (StringUtils.isNotBlank(fatherId) && level != null && order != null) {
            // 设定该顶事件的初始层级
            levelMap.put(fatherId, level);
            updateEdges.add(FtaEdge.builder().targetId(fatherId).edgeOrder(order).build());
        } else {
            topEvent = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId).eq(FtaNode::getTypeCode, FtaNodeEnum.顶事件.getIndex()));
            // 设定该顶事件的初始层级
            levelMap.put(topEvent.getId(), 1);
            fatherId = topEvent.getId();
        }
        orderMap.put(fatherId, 1);
        disposalLevelAneOrder(allEdge, middleNodeIds, fatherId, updateNodes, updateEdges, levelMap, orderMap, onlyLevel);
        if (!CollectionUtils.isEmpty(updateNodes)) {
            nodeMapper.updateNodes(updateNodes);
        }
        if (!CollectionUtils.isEmpty(updateEdges)) {
            updateNodes.clear();
            updateEdges.forEach(edge -> updateNodes.add(FtaNode.builder().id(edge.getTargetId()).edgeOrder(edge.getEdgeOrder()).build()));
            nodeMapper.updateNodes(updateNodes);
        }
    }

    /**
     * 递归组装新的节点层级和连线顺序
     *
     * @param allEdge       所有连线
     * @param middleNodeIds 所有中间事件节点id
     * @param fatherId      从该父节点id下递归筛选
     * @param updateNodes   待更新的数据
     * @param levelMap      层级map，必须先指定父节点id对应的层级值
     * @param orderMap      顺序map
     * @param onlyLevel     是否只计算层级
     */
    public static void disposalLevelAneOrder(List<FtaEdge> allEdge, List<String> middleNodeIds, String fatherId, List<FtaNode> updateNodes, List<FtaEdge> updateEdges, Map<String, Integer> levelMap, Map<String, Integer> orderMap, boolean onlyLevel) {
        // 父节点的层级
        updateNodes.add(FtaNode.builder().id(fatherId).nodeLevel(levelMap.get(fatherId)).build());
        for (FtaEdge edge : allEdge) {
            // 从父id往下找
            if (fatherId.equals(edge.getSourceId())) {
                // 当这条线连接的是 门->中间事件 时
                if (middleNodeIds.contains(edge.getTargetId())) {
                    levelMap.put(edge.getTargetId(), levelMap.get(fatherId) + 1);
                    if (!onlyLevel) {
                        orderMap.put(edge.getSourceId(), orderMap.get(edge.getSourceId()) == null ? 1 : orderMap.get(edge.getSourceId()) + 1);
                        updateEdges.add(FtaEdge.builder().targetId(edge.getTargetId()).edgeOrder(orderMap.get(edge.getSourceId())).build());
                    }
                } else {
                    levelMap.put(edge.getTargetId(), levelMap.get(fatherId));
                }
                disposalLevelAneOrder(allEdge, middleNodeIds, edge.getTargetId(), updateNodes, updateEdges, levelMap, orderMap, onlyLevel);
            }
        }
    }

    /**
     * 批量更新底事件计算参数 或 表决门计算参数 或 中间事件和顶事件的名称和编码
     *
     * @param parms
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> update(List<FtaNode> parms, HeaderUserInfo userInfo) {
        ArrayList<FtaNode> updateFathers = new ArrayList<>();
        ArrayList<FtaNode> updateNodes = new ArrayList<>();
        ArrayList<FtaNode> updateMirrors = new ArrayList<>();
        ArrayList<String> validMirrorIds = new ArrayList<>();
        parms.forEach(node -> {
            if (FtaNodeEnum.重复事件.getIndex().equals(node.getTypeCode()) && !validMirrorIds.contains(node.getMasterId())) {
                updateMirrors.add(node);
                validMirrorIds.add(node.getMasterId());
            } else {
                updateFathers.add(FtaNode.builder().id(node.getFatherId()).code(node.getCode()).name(node.getName()).build());
                FtaNode update = FtaNode.builder().id(node.getId()).build();
                if (StringUtils.isNotBlank(node.getDistributionTypeCode()) && node.getP1Value() != null) {
                    update.setDistributionTypeCode(node.getDistributionTypeCode());
                    update.setP1Value(node.getP1Value());
                    update.setP2Value(node.getP2Value());
                    update.setP3Value(node.getP3Value());
                    updateNodes.add(update);
                }
                if (node.getRequiredPath() != null) {
                    update.setRequiredPath(node.getRequiredPath() > 0 ? node.getRequiredPath() : null);
                    updateNodes.add(update);
                }
            }
        });
        updateNodes.addAll(updateFathers);
        if (!CollectionUtils.isEmpty(updateNodes)) {
            nodeMapper.updateNodes(updateNodes);
        }
        if (!CollectionUtils.isEmpty(updateMirrors)) {
            // 更新重复事件时，需要同时更新这个重复事件关联的所有重复事件的相关参数
            nodeMapper.updateMirrorNodes(updateMirrors);
            nodeMapper.updateMirrorFatherNodes(updateMirrors);
        }
        //清除历史计算数据
        String analysisId = nodeMapper.selectById(parms.get(0).getId()).getAnalysisId();
        String projectId = analysisMapper.selectOne(new LambdaQueryWrapper<FtaAnalysis>()
                .select(FtaAnalysis::getProjectId).eq(FtaAnalysis::getAnalysisId, analysisId)).getProjectId();
        clearCalculateHistory(userInfo, analysisId, projectId);
        return ServerResponse.createBySuccess();
    }

    /**
     * 根据节点id删除该节点及该节点下的所有子集
     *
     * @param nodeId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse delete(String nodeId, HeaderUserInfo userInfo) {
        FtaNode old = nodeMapper.selectById(nodeId);
        FtaNode middleNode = nodeMapper.selectById(old.getFatherId());
        JSONObject result = new JSONObject();
        //清除历史计算数据
        String projectId = analysisMapper.selectOne(new LambdaQueryWrapper<FtaAnalysis>()
                .select(FtaAnalysis::getProjectId).eq(FtaAnalysis::getAnalysisId, old.getAnalysisId())).getProjectId();
        clearCalculateHistory(userInfo, old.getAnalysisId(), projectId);
        // 获取该fta分析的所有连线
        List<FtaNode> nodes = nodeMapper.selectList(new LambdaUpdateWrapper<FtaNode>().eq(FtaNode::getAnalysisId, old.getAnalysisId()));
        List<FtaEdge> edges = generateEdgesByNodes(nodes);
        // 需要删除的所有id
        List<String> delNodeIds = new ArrayList<>();
        List<String> delEdgeIds = new ArrayList<>();
        if (FtaNodeEnum.顶事件.getIndex().equals(middleNode.getTypeCode())) {
            // 父节点是顶事件时，不能删除
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        } else if (FtaNodeEnum.与门.getIndex().equals(old.getTypeCode())
                || FtaNodeEnum.或门.getIndex().equals(old.getTypeCode())
                || FtaNodeEnum.表决门.getIndex().equals(old.getTypeCode())) {
            // 删除门，需要删除该门及子集，并删除子集有关联的重复事件和相关联的子分析
            // 找到该门及该门下 需要删除的节点id
            getRelevanceIdByNodeId(edges, old.getFatherId(), delNodeIds);
            // 找出需要删除的线
            getEdgeIdsByNodeIds(delEdgeIds, edges, delNodeIds);
            // 删除该门及该门下的数据，需要考虑转移门和额外删除的重复事件及连线，并记录删除操作用于回退
            deleteGate(userInfo, result, delNodeIds, delEdgeIds, projectId, old, middleNode, true);
        } else if (FtaNodeEnum.转移门.getIndex().equals(old.getTypeCode())) {
            // 删除转移门
            // 找到需要删除的节点id
            delNodeIds.add(nodeId);
            delNodeIds.add(old.getFatherId());
            // 找出需要删除的线
            getEdgeIdsByNodeIds(delEdgeIds, edges, delNodeIds);

            ArrayList<String> delAnalysisIds = new ArrayList<>();
            List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(projectId, userInfo.getUserId());
            delAnalysisIds.add(old.getChildrenAnalysisId());
            FtaAnalysisService.getChildrenAnalysisIds(old.getChildrenAnalysisId(), delAnalysisIds, allAnalysis);
            // 记录需要删除的底事件和连线
            operatorService.delete(null, userInfo, delNodeIds, delEdgeIds, middleNode.getId(), middleNode.getEdgeOrder(), old.getAnalysisId(), delAnalysisIds, true);
            analysisMapper.deleteBatchIds(delAnalysisIds);
            // 需要删除的转移门相关联的分析中的重复事件
            List<String> queryMasterIds = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                            .select(FtaNode::getMasterId)
                            .eq(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex())
                            .in(FtaNode::getAnalysisId, delAnalysisIds))
                    .stream().map(FtaNode::getMasterId).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(queryMasterIds)) {
                // 如果被删除的数据中存在重复事件
                // 找到这些重复事件相关联的不在需要删除的数据中的其它重复事件
                List<FtaNode> updateMirrorNodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                        .eq(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex())
                        .notIn(FtaNode::getAnalysisId, delAnalysisIds)
                        .in(FtaNode::getMasterId, queryMasterIds));
                // 将同一个masterId只剩下一个的重复事件更新为底事件
                updateExtraMirrors(updateMirrorNodes);
            }
            // 更新被删除的节点的父节那一层级右侧其它的连线顺序
            nodeMapper.moveLeftNotIns(middleNode.getFatherId(), delNodeIds, middleNode.getEdgeOrder());
            // 删除该转移门及相关连线
            nodeMapper.deleteBatchIds(delNodeIds);
            result.put(FtaConstants.REFRESH_ANALYSIS_TREE, true);
        } else if (FtaNodeEnum.重复事件.getIndex().equals(old.getTypeCode())
                || FtaNodeEnum.未展开事件.getIndex().equals(old.getTypeCode())) {
            // 删除重复事件和未展开事件
            // 需要一起删除上方的中间事件
            delNodeIds.add(nodeId);
            delNodeIds.add(old.getFatherId());
            // 找出需要删除的线
            getEdgeIdsByNodeIds(delEdgeIds, edges, old);
            if (FtaNodeEnum.重复事件.getIndex().equals(old.getTypeCode())) {
                // 删除重复事件时，如果删掉这个重复事件后，相关联的重复事件只剩一个，则将最后一个重复事件转为底事件
                List<FtaNode> mirrors = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                        .select(FtaNode::getId, FtaNode::getMasterId)
                        .eq(FtaNode::getMasterId, old.getMasterId()));
                if (mirrors.size() == 2) {
                    // 将最后一个重复事件更新为底事件
                    FtaNode lastMirror = mirrors.stream().filter(o -> !o.getId().equals(old.getId())).findFirst().orElse(new FtaNode());
                    nodeMapper.update(FtaNode.builder().typeCode(FtaNodeEnum.底事件.getIndex()).build(),
                            new LambdaUpdateWrapper<FtaNode>().eq(FtaNode::getId, lastMirror.getId()));
                }
            }
            nodeMapper.deleteBatchIds(delNodeIds);
            // 更新被删除的节点的父节那一层级右侧其它的连线顺序
            nodeMapper.moveLeftNotIn(middleNode.getFatherId(), middleNode.getId(), middleNode.getEdgeOrder());
            // 记录删除重复事件和未展开事件操作
            operatorService.delete(null, userInfo, delNodeIds, delEdgeIds, middleNode.getId(), middleNode.getEdgeOrder(), old.getAnalysisId(), null, true);
        } else if (FtaNodeEnum.底事件.getIndex().equals(old.getTypeCode())) {
            // 删除底事件
            // 需要一起删除上方的中间事件
            delNodeIds.add(nodeId);
            delNodeIds.add(old.getFatherId());
            // 找出需要删除的线
            getEdgeIdsByNodeIds(delEdgeIds, edges, delNodeIds);
            // 记录需要删除的底事件和连线
            operatorService.delete(null, userInfo, delNodeIds, delEdgeIds, middleNode.getId(), middleNode.getEdgeOrder(), old.getAnalysisId(), null, true);
            // 更新被删除的节点的父节那一层级右侧其它的连线顺序
            nodeMapper.moveLeftNotIns(middleNode.getFatherId(), delNodeIds, middleNode.getEdgeOrder());
            nodeMapper.deleteBatchIds(delNodeIds);
//            edgeMapper.deleteBatchIds(delEdgeIds);
        }
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 删除一个门下的数据。（并记录删除操作便于回退）
     *
     * @param userInfo
     * @param result       记录了是否需要刷新分析树
     * @param delNodeIds   需要删除的节点id
     * @param delEdgeIds   需要删除的连线id
     * @param projectId    当前分析projectId
     * @param old          被直接操作的那个门（删除该门下的数据）
     * @param fatherNode   上述门的父中间事件
     * @param isDeleteGate true：当前操作为删除门；false：当前操作为将门转为底事件，仅需删除门下的数据，不包括该门及父中间事件
     */
    private void deleteGate(HeaderUserInfo userInfo, JSONObject result, List<String> delNodeIds, List<String> delEdgeIds, String projectId, FtaNode old, FtaNode fatherNode, boolean isDeleteGate) {
        if (!CollectionUtils.isEmpty(delNodeIds) && !CollectionUtils.isEmpty(delEdgeIds)) {
            // 如果该门下包含转移门，则删除关联的分析
            List<FtaNode> delTransferNodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                    .in(FtaNode::getId, delNodeIds)
                    .eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex()));
            // 关联的子分析中的底事件
            List<String> queryMasterIds = new ArrayList<>();
            ArrayList<String> delAnalysisIds = new ArrayList<>();
            if (!CollectionUtils.isEmpty(delTransferNodes)) {
                // 找到转移门相关联的分析
                List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(projectId, userInfo.getUserId());
                delTransferNodes.forEach(node -> {
                    delAnalysisIds.add(node.getChildrenAnalysisId());
                    FtaAnalysisService.getChildrenAnalysisIds(node.getChildrenAnalysisId(), delAnalysisIds, allAnalysis);
                });
                analysisMapper.deleteBatchIds(delAnalysisIds);
                // 需要删除的转移门中相关联的分析中的重复事件节点的masterId
                queryMasterIds.addAll(nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                                .select(FtaNode::getMasterId)
                                .eq(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex())
                                .in(FtaNode::getAnalysisId, delAnalysisIds))
                        .stream().map(FtaNode::getMasterId).collect(Collectors.toList()));
                result.put(FtaConstants.REFRESH_ANALYSIS_TREE, true);
            }
            if (isDeleteGate) {
                // 记录删除门的操作
                operatorService.delete(null, userInfo, delNodeIds, delEdgeIds, fatherNode.getId(), fatherNode.getEdgeOrder(), old.getAnalysisId(), delAnalysisIds, true);
            } else {
                // 记录门转为底事件的操作
                operatorService.gateToCircle(null, userInfo, delNodeIds, delEdgeIds, old, old.getAnalysisId(), delAnalysisIds);
            }
            // 删除重复事件时，需要判断这些重复事件是否还存在关联的或者的重复事件，如果只剩一个存活，则需要将该重复事件转为底事件
            // 并且这些重复事件必须在被删除的子集和被删除的分析之外
            // 需要删除的所有的重复事件的masterId
            queryMasterIds.addAll(nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                            .select(FtaNode::getMasterId)
                            .in(FtaNode::getId, delNodeIds)
                            .eq(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex()))
                    .stream().map(FtaNode::getMasterId).collect(Collectors.toList()));
            queryMasterIds = queryMasterIds.stream().distinct().collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(queryMasterIds)) {
                // 如果被删除的数据中存在重复事件
                // 找到这些重复事件相关联的重复事件，且不在需要删除的数据中
                LambdaQueryWrapper<FtaNode> qw = new LambdaQueryWrapper<FtaNode>()
                        .eq(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex())
                        .in(FtaNode::getMasterId, queryMasterIds)
                        .notIn(FtaNode::getId, delNodeIds);
                if (!CollectionUtils.isEmpty(delAnalysisIds)) {
                    qw.notIn(FtaNode::getAnalysisId, delAnalysisIds);
                }
                List<FtaNode> mirrorNodes = nodeMapper.selectList(qw);
                // 将同一个masterId只剩下一个的重复事件更新为底事件
                updateExtraMirrors(mirrorNodes);
            }
            if (isDeleteGate) {
                // 删除门的操作需要更新
                // 更新被删除的节点的父节那一层级右侧其它的连线顺序
                nodeMapper.moveLeftNotIns(fatherNode.getId(), delNodeIds, fatherNode.getEdgeOrder());
            }
            // 删除该转移门及相关连线
            nodeMapper.deleteBatchIds(delNodeIds);
        }
    }

    /**
     * 删除子集或转移门时，如果这些数据删除后，同一个masterId只剩下一个重复事件，则需要将这个重复事件更新为底事件
     *
     * @param mirrorNodes 需要被额外更新的重复事件
     */
    private void updateExtraMirrors(List<FtaNode> mirrorNodes) {
        if (!CollectionUtils.isEmpty(mirrorNodes)) {
            ArrayList<FtaNode> updateMirrors = new ArrayList<>();
            mirrorNodes.stream().collect(Collectors.groupingBy(FtaNode::getMasterId))
                    .forEach((masterId, list) -> {
                        if (list.size() == 1) {
                            // 只剩一个关联的重复事件，记录这个重复事件的id，之后将其更新为底事件
                            updateMirrors.add(list.get(0));
                        }
                    });
            if (!CollectionUtils.isEmpty(updateMirrors)) {
                // 将同一个masterId的最后一个重复事件转为底事件，只改变节点类型，不清除masterId，便于后续撤销操作
                nodeMapper.updateTypeCodes(updateMirrors, FtaNodeEnum.底事件.getIndex());
            }
        }
    }

    /**
     * 根据projectId查出所有分析Id
     *
     * @param projectId
     * @param userName
     * @return
     */
    List<String> getAllAnalysisIdByProjectId(String projectId, String userName) {
        return analysisMapper.selectList(new LambdaQueryWrapper<FtaAnalysis>()
                        .select(FtaAnalysis::getAnalysisId)
                        .eq(FtaAnalysis::getProjectId, projectId)
                        .eq(FtaAnalysis::getCreateBy, userName))
                .stream().map(FtaAnalysis::getAnalysisId).collect(Collectors.toList());
    }

    /**
     * 根据projectId查出所有分析
     *
     * @param projectId
     * @param userName
     * @return
     */
    List<FtaAnalysis> getAllAnalysisByProjectId(String projectId, String userName) {
        return analysisMapper.selectList(new LambdaQueryWrapper<FtaAnalysis>()
                .select(FtaAnalysis::getAnalysisId, FtaAnalysis::getFatherId)
                .eq(FtaAnalysis::getProjectId, projectId)
                .eq(FtaAnalysis::getCreateBy, userName));
    }

    /**
     * 剪切中间事件及子集（包含转移门）到一个门下
     *
     * @param sourceMiddleId 被剪切的中间事件id
     * @param targetMiddleId 目标顶事件id或中间事件id，剪切到该事件下面的门下
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse nodeCut(HeaderUserInfo userInfo, String sourceMiddleId, String targetMiddleId, boolean operator) {
        FtaNode sourceMiddleNode = nodeMapper.selectById(sourceMiddleId);
        String analysisId = sourceMiddleNode.getAnalysisId();
        FtaNode targetGate = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getAnalysisId, analysisId)
                .eq(FtaNode::getFatherId, targetMiddleId));
        if (StringUtils.isBlank(analysisId) || targetGate == null) {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请先选择门上方的名称框再进行粘贴, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        } else if (!sourceMiddleNode.getAnalysisId().equals(targetGate.getAnalysisId())) {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.不支持跨分析粘贴, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
        FtaNode sourceChildrenNode = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getAnalysisId, analysisId)
                .eq(FtaNode::getFatherId, sourceMiddleNode.getId()));
        String targetGateId = targetGate.getId();
        // 被剪切的只能是中间事件，只能复制到基本门下
        if (FtaNodeEnum.中间事件.getIndex().equals(sourceMiddleNode.getTypeCode())
                || !(FtaNodeEnum.与门.getIndex().equals(targetGate.getTypeCode())
                || FtaNodeEnum.或门.getIndex().equals(targetGate.getTypeCode())
                || FtaNodeEnum.表决门.getIndex().equals(targetGate.getTypeCode()))) {
            if (FtaNodeEnum.重复事件.getIndex().equals(sourceChildrenNode.getTypeCode())) {
                // 被剪切的是重复事件时，同一个父中间事件下不能存在相同的重复事件（也包括被重复的底事件）
                List<String> ids = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                                .select(FtaNode::getId)
                                .eq(FtaNode::getAnalysisId, analysisId)
                                .eq(FtaNode::getFatherId, targetGate.getId()))
                        .stream().map(FtaNode::getId).collect(Collectors.toList());
                Long count = nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>()
                        .eq(FtaNode::getAnalysisId, analysisId)
                        .in(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex(), FtaNodeEnum.底事件.getIndex())
                        .in(FtaNode::getFatherId, ids)
                        .and(o -> o.eq(FtaNode::getMasterId, sourceChildrenNode.getMasterId()).or().eq(FtaNode::getId, sourceChildrenNode.getMasterId())));
                if (!CollectionUtils.isEmpty(ids) && count > 0) {
                    throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.同一个门下禁止添加同一个重复事件, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
                }
            }
            // 获取该fta分析的所有连线
            List<FtaNode> nodes = nodeMapper.selectList(new LambdaUpdateWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId));
            List<FtaEdge> allEdges = generateEdgesByNodes(nodes);
            // 被剪切的事件，不能粘贴到该事件或该事件的子集中
            ArrayList<String> validIds = new ArrayList<>();
            getRelevanceIdByNodeId(allEdges, sourceMiddleId, validIds);
            if (validIds.contains(targetMiddleId)) {
                // 目标事件在被剪切事件的子集中
                throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请勿将一个事件剪切到该事件的子集中, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
            }
            //清除历史计算数据
            String projectId = analysisMapper.selectOne(new LambdaQueryWrapper<FtaAnalysis>()
                    .select(FtaAnalysis::getProjectId).eq(FtaAnalysis::getAnalysisId, analysisId)).getProjectId();
            clearCalculateHistory(userInfo, analysisId, projectId);
            // 更新被剪切的节点的父节点下其它的连线顺序
            nodeMapper.moveLeft(sourceMiddleNode.getFatherId(), sourceMiddleNode.getAnalysisId(), sourceMiddleNode.getEdgeOrder());
            // 被剪切的中间事件需要重新设定顺序
            Long edgeOrder = nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>()
                    .eq(FtaNode::getFatherId, targetGateId));
            // 计算层级差值
            int level = 0;
            if (sourceMiddleNode.getNodeLevel() != null && targetGate.getNodeLevel() != null) {
                Integer sourceFatherLevel = sourceMiddleNode.getNodeLevel() - 1;
                level = sourceFatherLevel - targetGate.getNodeLevel();
            }
            // 更新该节点的父id
            nodeMapper.updateById(FtaNode.builder().id(sourceMiddleId).fatherId(targetGateId).edgeOrder((int) (edgeOrder + 1)).build());
            // 更新节点父id上面的一条连线
            // 更新该节点及子集的层级
            ArrayList<String> updateNodeIds = new ArrayList<>();
            // 获取该节点下的子集id（包含该节点）
            getRelevanceIdByNodeId(allEdges, sourceMiddleNode.getId(), updateNodeIds);
            // 更新这些节点的层级
            List<FtaNode> updateNodeLevels = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                    .select(FtaNode::getId, FtaNode::getNodeLevel)
                    .eq(FtaNode::getAnalysisId, analysisId).in(FtaNode::getId, updateNodeIds));
            Integer finalLevel = level;
            updateNodeLevels.forEach(node -> node.setNodeLevel(node.getNodeLevel() - finalLevel));
            nodeMapper.updateNodes(updateNodeLevels);
            if (operator) {
                // 记录剪切操作，用于回退（回退时依然会调用此方法，此时不需要记录）
                operatorService.cutNodes(userInfo, JSON.toJSONString(FtaEdge.builder().sourceId(sourceMiddleId).targetId(nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getId, sourceMiddleNode.getFatherId())).getFatherId()).edgeOrder(sourceMiddleNode.getEdgeOrder()).build()), analysisId);
            }
            return ServerResponse.createBySuccess();
        }
        throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请先选择门上方的名称框再进行粘贴, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
    }

    /**
     * 复制中间事件（或顶事件）及子集（包含转移门）到一个门下，并复制子集所关联的子树
     *
     * @param sourceMiddleId  被复制的中间事件（或顶事件）id
     * @param targetMiddleId  目标顶事件id或中间事件id，复制到该事件下面的门下
     * @param isPasteToMirror 是否为底事件粘贴为重复事件
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse nodeCopy(HeaderUserInfo userInfo, String sourceMiddleId, String targetMiddleId, boolean isPasteToMirror) {
        long l = System.currentTimeMillis();
        FtaNode middleNode = nodeMapper.selectById(sourceMiddleId);
        String analysisId = middleNode.getAnalysisId();
        FtaNode targetGate = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getAnalysisId, analysisId)
                .eq(FtaNode::getFatherId, targetMiddleId));
        // 被复制的只能是中间事件或顶事件，只能复制到基本门下
        if (StringUtils.isBlank(analysisId) || targetGate == null
                || !(FtaNodeEnum.中间事件.getIndex().equals(middleNode.getTypeCode())
                || FtaNodeEnum.顶事件.getIndex().equals(middleNode.getTypeCode()))
                || !(FtaNodeEnum.与门.getIndex().equals(targetGate.getTypeCode())
                || FtaNodeEnum.或门.getIndex().equals(targetGate.getTypeCode())
                || FtaNodeEnum.表决门.getIndex().equals(targetGate.getTypeCode()))) {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请先选择门上方的名称框再进行粘贴, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        } else if (!middleNode.getAnalysisId().equals(targetGate.getAnalysisId())) {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.不支持跨分析粘贴, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
        String projectId = analysisMapper.selectOne(new LambdaQueryWrapper<FtaAnalysis>()
                .select(FtaAnalysis::getProjectId).eq(FtaAnalysis::getAnalysisId, analysisId)).getProjectId();
        String gateId = targetGate.getId();
        FtaNode sourceChildrenNode = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getAnalysisId, analysisId)
                .eq(FtaNode::getFatherId, middleNode.getId()));
        if (FtaNodeEnum.重复事件.getIndex().equals(sourceChildrenNode.getTypeCode()) || isPasteToMirror) {
            // 复制的是重复事件 或 将底事件粘贴为重复事件 时，同一个父中间事件下不能存在相同的重复事件（也包括被重复的底事件）
            List<String> ids = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                            .select(FtaNode::getId)
                            .eq(FtaNode::getAnalysisId, analysisId)
                            .eq(FtaNode::getFatherId, targetGate.getId()))
                    .stream().map(FtaNode::getId).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(ids)) {
                Long count = nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>()
                        .eq(FtaNode::getAnalysisId, analysisId)
                        .in(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex(), FtaNodeEnum.底事件.getIndex())
                        .in(FtaNode::getFatherId, ids)
                        .and(o -> o.eq(FtaNode::getMasterId, sourceChildrenNode.getMasterId()).or().eq(FtaNode::getId, sourceChildrenNode.getMasterId())));
                if (count > 0) {
                    throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.同一个门下禁止添加同一个重复事件, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
                }
            }
        }
        // 被复制的中间事件需要重新设定顺序
        Long edgeOrder = nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getFatherId, gateId));
        // 计算层级差值
        int level = 0;
        if (middleNode.getNodeLevel() != null && targetGate.getNodeLevel() != null) {
            Integer sourceFatherLevel = middleNode.getNodeLevel() - 1;
            level = sourceFatherLevel - targetGate.getNodeLevel();
        }
        ArrayList<String> copyNodeIds = new ArrayList<>();
        ArrayList<String> copyEdgeIds = new ArrayList<>();
        // 获取该fta分析的所有连线和节点
        List<FtaNode> nodes = nodeMapper.selectList(new LambdaUpdateWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId));
        List<FtaEdge> allEdges = generateEdgesByNodes(nodes);
        // 需要对新的数据进行重新编号
        String numberStr = "%04d";
        final int[] number = {Integer.parseInt(getEventCode(middleNode.getAnalysisId(), userInfo).getData())};
        if (String.valueOf(number[0]).length() > 4) {
            // 超长
            numberStr = "%0" + String.valueOf(number[0]).length() + "d";
        }
        // 获取该节点下的子集id（需要复制的id）
        getRelevanceIdByNodeId(allEdges, middleNode.getId(), copyNodeIds);
        getEdgeIdsByNodeIds(copyEdgeIds, allEdges, copyNodeIds);
        // 需要更新的节点和连线
        List<FtaNode> needUpdateNodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>().in(FtaNode::getId, copyNodeIds));
        // 如果该中间事件的子集中已经包含了转移门，则复制这些转移门关联的父分析id及所有子集分析的id
        // 找到需要复制的子分析集合
        List<FtaAnalysis> allAnalysis = analysisMapper.selectList(
                new LambdaQueryWrapper<FtaAnalysis>()
                        .eq(FtaAnalysis::getProjectId, projectId)
                        .eq(FtaAnalysis::getCreateBy, userInfo.getUserName()));
        ArrayList<String> copyAnalysisIds = new ArrayList<>();
        needUpdateNodes.forEach(node -> {
            if (FtaNodeEnum.转移门.getIndex().equals(node.getTypeCode())
                    && copyNodeIds.contains(node.getId())) {
                allAnalysis.forEach(item -> {
                    if (node.getChildrenAnalysisId().equals(item.getAnalysisId())) {
                        copyAnalysisIds.add(item.getAnalysisId());
                    }
                });
            }
        });
        HashMap<String, String> oldNodeIdAndNewMap = new HashMap<>();
        ArrayList<FtaAnalysis> insertAnalysis = new ArrayList<>();
        ArrayList<FtaNode> insertNodes = new ArrayList<>();
        ArrayList<FtaEdge> insertEdges = new ArrayList<>();
        // 复制子分析
        HashMap<String, String> analysisIdMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(copyAnalysisIds)) {
            String faherAnalysisId = middleNode.getAnalysisId();
            copyAnalysisIds.forEach(copyAnalysisId -> copyAnalysis(copyAnalysisId, userInfo, faherAnalysisId, analysisIdMap, oldNodeIdAndNewMap, insertAnalysis, insertNodes, insertEdges));
        }
        // 复制本分析中的子集
        // 大数据量批量插入
        Integer finalLevel = level;
        String finalNumberStr = numberStr;
        needUpdateNodes.forEach(item -> {
            if (isPasteToMirror && sourceChildrenNode.getId().equals(item.getId())) {
                // 底事件复制为重复事件时，设置新复制出来的节点为重复事件
                item.setTypeCode(FtaNodeEnum.重复事件.getIndex());
                item.setMasterId(StringUtils.isNotBlank(item.getMasterId()) ? item.getMasterId() : item.getId());
                // 更新这个原来的底事件为重复事件
                nodeMapper.updateById(FtaNode.builder().id(item.getId())
                        .masterId(StringUtils.isNotBlank(item.getMasterId()) ? item.getMasterId() : item.getId())
                        .typeCode(FtaNodeEnum.重复事件.getIndex()).build());
            } else if (!FtaNodeEnum.重复事件.getIndex().equals(item.getTypeCode())) {
                // 非重复事件时，重置masterId
                item.setMasterId(null);
            }
            item.setChildrenAnalysisId(StringUtils.isNotBlank(item.getChildrenAnalysisId()) && StringUtils.isNotBlank(analysisIdMap.get(item.getChildrenAnalysisId())) ? analysisIdMap.get(item.getChildrenAnalysisId()) : StringUtils.EMPTY);
            if (sourceMiddleId.equals(item.getId())) {
                // 更新新的中间事件的顺序
                item.setEdgeOrder(Math.toIntExact(edgeOrder + 1));
            }
            String oldId = item.getId();
            item.setId(UUIDUtil.getUuid());
            item.setQ(null);
            item.setNodeLevel(item.getNodeLevel() - finalLevel);
            insertNodes.add(item);
            oldNodeIdAndNewMap.put(oldId, item.getId());
        });
        oldNodeIdAndNewMap.put(middleNode.getFatherId(), gateId);
        // 获取转移门数据，统一转移门与子分析的名称和编码
        HashMap<String, String> analysisIdAndFatherTransferGateFatherId = new HashMap<>();
        HashMap<String, String> nodeIdAndCode = new HashMap<>();
        // 组装父节点id，插入新的节点数据 设置新的编码
        // 不需要重新设置code的重复事件的父id
        ArrayList<String> valid = new ArrayList<>();
        insertNodes.forEach(node -> {
            // 更新节点的fatherId
            node.setFatherId(StringUtils.isNotBlank(oldNodeIdAndNewMap.get(node.getFatherId())) ? oldNodeIdAndNewMap.get(node.getFatherId()) : "0");
            if (FtaNodeEnum.重复事件.getIndex().equals(node.getTypeCode()) && !isPasteToMirror) {
                // 不需要重新设置code的重复事件的父中间事件id
                valid.add(node.getFatherId());
            }
        });
        List<FtaNode> insertNodesSorted = insertNodes.stream().sorted(Comparator.comparing(FtaNode::getAnalysisId).thenComparing(FtaNode::getNodeLevel)).collect(Collectors.toList());
        insertNodesSorted.forEach(node -> {
            if ((FtaNodeEnum.中间事件.getIndex().equals(node.getTypeCode())
                    || node.getId().equals(oldNodeIdAndNewMap.get(sourceMiddleId))) && !valid.contains(node.getId())) {
                // 设置新的编码
                node.setCode(FtaConstants.EVENT_CODE + String.format(finalNumberStr, number[0]++));
            }
            if (isPasteToMirror && node.getId().equals(oldNodeIdAndNewMap.get(sourceMiddleId))) {
                // 底事件复制为重复事件时，统一重复事件code
                node.setCode(middleNode.getCode());
            }
            // 如果被复制的是顶事件，更新这个顶事件的typeCode为中间事件
            if (FtaNodeEnum.顶事件.getIndex().equals(middleNode.getTypeCode()) && node.getId().equals(oldNodeIdAndNewMap.get(sourceMiddleId))) {
                node.setTypeCode(FtaNodeEnum.中间事件.getIndex());
            }
            nodeIdAndCode.put(node.getId(), node.getCode());
            analysisIdAndFatherTransferGateFatherId.put(node.getChildrenAnalysisId(), node.getFatherId());
        });
        // 统一转移门与子分析的名称和编码
        insertNodesSorted.forEach(node -> {
            if (FtaNodeEnum.顶事件.getIndex().equals(node.getTypeCode()) && !node.getId().equals(oldNodeIdAndNewMap.get(sourceMiddleId))) {
                node.setCode(nodeIdAndCode.get(analysisIdAndFatherTransferGateFatherId.get(node.getAnalysisId())));
            }
        });
        insertAnalysis.forEach(newAnalysis -> newAnalysis.setAnalysisName(nodeIdAndCode.get(analysisIdAndFatherTransferGateFatherId.get(newAnalysis.getAnalysisId()))));
        if (!CollectionUtils.isEmpty(insertAnalysis)) {
            analysisMapper.inserts(insertAnalysis);
        }
        nodeService.saveBatch(insertNodes);
        // 顶事件上面是没有数据的，如果被复制的是顶事件，需要额外增加一条连线
        String newEdgeId = UUIDUtil.getUuid();
        JSONObject result = new JSONObject();
        if (!CollectionUtils.isEmpty(copyAnalysisIds)) {
            result.put(FtaConstants.REFRESH_ANALYSIS_TREE, true);
        }
        // 记录复制分析的操作
        List<String> insertEdgeIds = insertEdges.stream().map(FtaEdge::getId).collect(Collectors.toList());
        if (FtaNodeEnum.顶事件.getIndex().equals(middleNode.getTypeCode())) {
            insertEdgeIds.add(newEdgeId);
        }
        operatorService.insert(userInfo, insertNodesSorted.stream().map(FtaNode::getId).collect(Collectors.toList()), insertEdgeIds, analysisId, insertAnalysis.stream().map(FtaAnalysis::getAnalysisId).collect(Collectors.toList()), FtaOperatorTypeEnum.复制.getIndex());
        //清除历史计算数据
        clearCalculateHistory(userInfo, analysisId, projectId);
        log.warn("复制耗时：" + (System.currentTimeMillis() - l));
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 复制分析到指定的父分析id下
     *
     * @param analysisId
     * @param userInfo
     * @param faherAnalysisId
     * @return
     */
    private void copyAnalysis(String analysisId, HeaderUserInfo userInfo, String faherAnalysisId, Map<String, String> analysisIdMap, Map<String, String> oldNodeIdAndNewMap, List<FtaAnalysis> insertAnalysis, List<FtaNode> insertNodes, List<FtaEdge> insertEdges) {
        FtaAnalysis analysis = analysisMapper.selectById(analysisId);
        //复制节点信息：待完善
        List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(analysis.getProjectId(), userInfo.getUserId());
        List<FtaNode> nodes = nodeMapper.selectList(
                new LambdaQueryWrapper<FtaNode>()
                        .eq(FtaNode::getAnalysisId, analysisId)
                        .eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex())
                        .isNotNull(FtaNode::getChildrenAnalysisId));
        // 获取需要更新的分析id及子树id
        ArrayList<String> analysisIds = new ArrayList<>();
        analysisIds.add(analysisId);
        nodes.forEach(node -> {
            analysisIds.add(node.getChildrenAnalysisId());
            FtaAnalysisService.getChildrenAnalysisIds(node.getChildrenAnalysisId(), analysisIds, allAnalysis);
        });
        //批量复制分析及该分析下的节点连线数据
        copyAnalysisAndNodes(analysisIds, faherAnalysisId, analysisIdMap, oldNodeIdAndNewMap, insertAnalysis, insertNodes, insertEdges);
    }

    /**
     * 批量复制分析
     *
     * @param analysisIds
     */
    private void copyAnalysisAndNodes(List<String> analysisIds, String faherAnalysisId, Map<String, String> analysisIdMap, Map<String, String> oldNodeIdAndNewMap, List<FtaAnalysis> insertAnalysis, List<FtaNode> insertNodes, List<FtaEdge> insertEdges) {
        // 需要新插入的所有分析
        List<FtaAnalysis> inserts = analysisMapper.selectList(
                new LambdaQueryWrapper<FtaAnalysis>()
                        .in(FtaAnalysis::getAnalysisId, analysisIds));
        inserts = inserts.stream().peek(analysis -> {
            String copyNewAnalysisId = BasicDataUtil.uuidGenerator();
            if (null != analysis.getLevel() && 1 == analysis.getLevel()) {
                analysis.setAnalysisName(analysis.getAnalysisName() + "_copy");
            }
            analysisIdMap.put(analysis.getAnalysisId(), copyNewAnalysisId);
            analysis.setAnalysisId(copyNewAnalysisId);
        }).collect(Collectors.toList());
        inserts.forEach(analysis -> {
            analysis.setFatherId(faherAnalysisId.equals(analysis.getFatherId()) ? faherAnalysisId : analysisIdMap.get(analysis.getFatherId()));
            analysis.setCreateTime(new Date());
        });
        insertAnalysis.addAll(inserts);
        // 需要更新的节点和连线
        List<FtaNode> allNodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>().in(FtaNode::getAnalysisId, analysisIds));
        List<FtaEdge> allEdges = generateEdgesByNodes(allNodes);
        // 批量插入
        allNodes.forEach(item -> {
            item.setAnalysisId(analysisIdMap.get(item.getAnalysisId()));
            item.setChildrenAnalysisId(StringUtils.isNotBlank(item.getChildrenAnalysisId()) ? analysisIdMap.get(item.getChildrenAnalysisId()) : StringUtils.EMPTY);
            String oldId = item.getId();
            item.setId(UUIDUtil.getUuid());
            item.setQ(null);
            oldNodeIdAndNewMap.put(oldId, item.getId());
            insertNodes.add(item);
        });
        allEdges.forEach(item -> {
            item.setAnalysisId(analysisIdMap.get(item.getAnalysisId()));
            item.setId(UUIDUtil.getUuid());
            item.setTargetId(oldNodeIdAndNewMap.get(item.getTargetId()));
            item.setSourceId(oldNodeIdAndNewMap.get(item.getSourceId()));
            insertEdges.add(item);
        });
    }

    /**
     * 根据fatherId的连线逐层找出其子集的所有关联的id（包含fatherId）
     *
     * @param allEdge
     * @param fatherId
     * @return
     */
    public static void getRelevanceIdByNodeId(List<FtaEdge> allEdge, String fatherId, List<String> delNodeIds) {
        delNodeIds.add(fatherId);
        for (FtaEdge edge : allEdge) {
            if (fatherId.equals(edge.getSourceId())) {
                getRelevanceIdByNodeId(allEdge, edge.getTargetId(), delNodeIds);
            }
        }
    }

    /**
     * 找出node及nodeFatherId关联的所有连线
     *
     * @param delEdgeIds
     * @param allEdge
     * @param node
     */
    public static void getEdgeIdsByNodeIds(List<String> delEdgeIds, List<FtaEdge> allEdge, FtaNode node) {
        allEdge.forEach(edge -> {
            if (edge.getSourceId().equals(node.getId())
                    || edge.getTargetId().equals(node.getId())
                    || edge.getSourceId().equals(node.getFatherId())
                    || edge.getTargetId().equals(node.getFatherId())) {
                delEdgeIds.add(edge.getId());
            }
        });
    }

    /**
     * 找出nodeIds关联的所有连线
     *
     * @param delEdgeIds
     * @param allEdge
     * @param nodeIds
     */
    public static void getEdgeIdsByNodeIds(List<String> delEdgeIds, List<FtaEdge> allEdge, List<String> nodeIds) {
        allEdge.forEach(edge -> {
            if (nodeIds.contains(edge.getSourceId())
                    || nodeIds.contains(edge.getTargetId())) {
                delEdgeIds.add(edge.getId());
            }
        });
    }

    /**
     * 找到该分析中所有的对应事件Id
     *
     * @param analysisId
     * @param userInfo
     * @return
     */
    public ServerResponse getMasterNodes(String analysisId, HeaderUserInfo userInfo) {
        ArrayList<JSONObject> result = new ArrayList<>();
        List<FtaNode> nodes;
        // 判断该分析是否包含父子分析
        if (analysisMapper.selectCount(
                new LambdaQueryWrapper<FtaAnalysis>().eq(FtaAnalysis::getFatherId, analysisId)) > 0
                || nodeMapper.selectCount(
                new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex())
                        .eq(FtaNode::getChildrenAnalysisId, analysisId)) > 0) {
            // 当该分析中包含父子分析，需要找到所有有关联的分析中的对应事件
            // 获取该分析id有关联的所有父子分析id
            FtaAnalysis analysis = analysisMapper.selectById(analysisId);

            List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(analysis.getProjectId(), userInfo.getUserId());
            ArrayList<String> analysisIds = new ArrayList<>();
            FtaAnalysisService.getAnalysisIds(analysisId, analysisIds, allAnalysis);
            nodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                    .in(FtaNode::getAnalysisId, analysisIds)
                    .and(o -> o.eq(FtaNode::getTypeCode, FtaNodeEnum.底事件.getIndex())
                            .or().eq(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex())));
        } else {
            // 当该分析中不包含父子分析，直接取值
            nodes = nodeMapper.selectList(
                    new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId)
                            .and(o -> o.eq(FtaNode::getTypeCode, FtaNodeEnum.底事件.getIndex())
                                    .or().eq(FtaNode::getTypeCode, FtaNodeEnum.重复事件.getIndex())));
        }
        Map<String, List<FtaNode>> map = null;
        if (!CollectionUtils.isEmpty(nodes)) {
            List<String> fatherIds = nodes.stream().map(FtaNode::getFatherId).collect(Collectors.toList());
            List<FtaNode> fatherNodes = nodeMapper.selectList(
                    new LambdaQueryWrapper<FtaNode>().in(FtaNode::getId, fatherIds)
            );
            map = fatherNodes.stream().collect(Collectors.groupingBy(FtaNode::getId));
        }
        Map<String, List<FtaNode>> finalMap = map;
        ArrayList<String> filter = new ArrayList<>();
        nodes.forEach(node -> {
            JSONObject js = new JSONObject();
            if (!filter.contains(StringUtils.isNotBlank(node.getMasterId()) ? node.getMasterId() : node.getId())) {
                js.put("id", StringUtils.isNotBlank(node.getMasterId()) ? node.getMasterId() : node.getId());
                js.put("code", !CollectionUtils.isEmpty(finalMap)
                        && !CollectionUtils.isEmpty(finalMap.get(node.getFatherId()))
                        ? finalMap.get(node.getFatherId()).get(0).getCode() : StringUtils.EMPTY);
                js.put("name", !CollectionUtils.isEmpty(finalMap)
                        && !CollectionUtils.isEmpty(finalMap.get(node.getFatherId()))
                        ? finalMap.get(node.getFatherId()).get(0).getName() : StringUtils.EMPTY);
                result.add(js);
                filter.add(StringUtils.isNotBlank(node.getMasterId()) ? node.getMasterId() : node.getId());
            }
        });
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 获取事件编码
     *
     * @param analysisId
     * @param userInfo
     * @return
     */
    public ServerResponse<String> getEventCode(String analysisId, HeaderUserInfo userInfo) {
        String numberStr = "%04d";
        List<String> codes;
        // 判断该分析是否包含父子分析
        if (analysisMapper.selectCount(
                new LambdaQueryWrapper<FtaAnalysis>().eq(FtaAnalysis::getFatherId, analysisId)) > 0
                || nodeMapper.selectCount(
                new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex())
                        .eq(FtaNode::getChildrenAnalysisId, analysisId)) > 0) {
            // 当该分析中包含父子分析，需要找到所有有关联的分析中的事件数量
            // 获取该分析id有关联的所有父子分析id
            FtaAnalysis analysis = analysisMapper.selectById(analysisId);
            List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(analysis.getProjectId(), userInfo.getUserId());
            ArrayList<String> analysisIds = new ArrayList<>();
            FtaAnalysisService.getAnalysisIds(analysisId, analysisIds, allAnalysis);

            codes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                            .select(FtaNode::getCode)
                            .in(FtaNode::getAnalysisId, analysisIds)
                            .in(FtaNode::getTypeCode, FtaNodeEnum.中间事件.getIndex(), FtaNodeEnum.顶事件.getIndex())
                            .likeRight(FtaNode::getCode, FtaConstants.EVENT_CODE))
                    .stream().map(FtaNode::getCode).collect(Collectors.toList());
        } else {
            // 当该分析中不包含父子分析
            codes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                            .select(FtaNode::getCode)
                            .eq(FtaNode::getAnalysisId, analysisId)
                            .in(FtaNode::getTypeCode, FtaNodeEnum.中间事件.getIndex(), FtaNodeEnum.顶事件.getIndex())
                            .likeRight(FtaNode::getCode, FtaConstants.EVENT_CODE))
                    .stream().map(FtaNode::getCode).collect(Collectors.toList());
        }
        final Integer[] count = {0};
        try {
            codes.forEach(code -> {
                String replace = code.replace(FtaConstants.EVENT_CODE, StringUtils.EMPTY);
                int num = Integer.parseInt(replace);
                if (num > count[0]) {
                    count[0] = num;
                }
            });
        } catch (Exception e) {
            log.error("事件编码:", e);
            return ServerResponse.createBySuccess(StringUtils.EMPTY);
        }
        count[0]++;
        if (String.valueOf(count[0]).length() > 4) {
            // 超长
            numberStr = "%0" + String.valueOf(count[0]).length() + "d";
        }
        return ServerResponse.createBySuccess(String.format(numberStr, count[0]));
    }

    /**
     * rollback
     *
     * @param userInfo
     * @param analysisId
     * @return
     */
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_UNCOMMITTED)
    public ServerResponse rollback(HeaderUserInfo userInfo, String analysisId) {
        // 查出最新的操作记录
        FtaOperator operator = operatorMapper.selectOne(new LambdaQueryWrapper<FtaOperator>()
                .eq(FtaOperator::getAnalysisId, analysisId)
                .isNull(FtaOperator::getFatherId)
                .orderByDesc(FtaOperator::getInsertTime).last(" LIMIT 1"));
        if (operator != null) {
            String projectId = analysisMapper.selectOne(new LambdaQueryWrapper<FtaAnalysis>()
                    .select(FtaAnalysis::getProjectId).eq(FtaAnalysis::getAnalysisId, analysisId)).getProjectId();
            ArrayList<Integer> delOperatorIds = new ArrayList<>();
            delOperatorIds.add(operator.getId());
            JSONObject result = new JSONObject();
            // 进行回退
            if (FtaOperatorTypeEnum.更新连线顺序.getIndex().equals(operator.getOperatorType()) && StringUtils.isNotBlank(operator.getReverseSql())) {
                // 执行速度慢，待完善
                runSql(operator.getReverseSql());
                operatorMapper.deleteById(operator.getId());
                return ServerResponse.createBySuccess();
            } else if (FtaOperatorTypeEnum.新增节点和连线.getIndex().equals(operator.getOperatorType())) {
                LambdaQueryWrapper<FtaOperatorDetail> qw = new LambdaQueryWrapper<FtaOperatorDetail>()
                        .eq(FtaOperatorDetail::getOperatorId, operator.getId());
                List<FtaOperatorDetail> details = operatorDetailMapper.selectList(qw);
                rollbackDetail(delOperatorIds, details, result, userInfo, false, null, projectId);
            } else if (FtaOperatorTypeEnum.复制.getIndex().equals(operator.getOperatorType())) {
                LambdaQueryWrapper<FtaOperatorDetail> qw = new LambdaQueryWrapper<FtaOperatorDetail>()
                        .eq(FtaOperatorDetail::getOperatorId, operator.getId());
                List<FtaOperatorDetail> details = operatorDetailMapper.selectList(qw);
                rollbackDetail(delOperatorIds, details, result, userInfo, false, null, projectId);
            } else if (FtaOperatorTypeEnum.剪切.getIndex().equals(operator.getOperatorType()) && StringUtils.isNotBlank(operator.getReverseSql())) {
                FtaEdge edge = JSON.parseObject(operator.getReverseSql(), FtaEdge.class);
                String sourceMiddleId = edge.getSourceId();
                String targetMiddleId = edge.getTargetId();
                Integer order = edge.getEdgeOrder();
                nodeCut(userInfo, sourceMiddleId, targetMiddleId, false);
                recoverOrder(sourceMiddleId, targetMiddleId, order);
                operatorMapper.deleteById(operator.getId());
            } else if (FtaOperatorTypeEnum.删除节点和连线.getIndex().equals(operator.getOperatorType())) {
                // 恢复主操作被删除的节点和连线
                LambdaQueryWrapper<FtaOperatorDetail> qw = new LambdaQueryWrapper<FtaOperatorDetail>()
                        .eq(FtaOperatorDetail::getOperatorId, operator.getId());
                List<FtaOperatorDetail> details = operatorDetailMapper.selectList(qw);
                rollbackDetail(delOperatorIds, details, result, userInfo, true, null, projectId);
                // 恢复主操作的节点顺序
                FtaNode middle = nodeMapper.selectById(operator.getRelateId());
                nodeMapper.moveRightNotIn(middle.getFatherId(), operator.getRelateId(), operator.getEdgeOrder());
            } else if (FtaOperatorTypeEnum.基本门转为底事件.getIndex().equals(operator.getOperatorType())) {
                // 此回退仅适用于基本门下有子集的情况（无子集的情况记录为 节点类型字段更新）
                // 恢复主操作被删除的节点和连线
                LambdaQueryWrapper<FtaOperatorDetail> qw = new LambdaQueryWrapper<FtaOperatorDetail>()
                        .eq(FtaOperatorDetail::getOperatorId, operator.getId());
                List<FtaOperatorDetail> details = operatorDetailMapper.selectList(qw);
                // 当前恢复的是主操作时，需要恢复被转换的节点类型，存储在了主操作中
                rollbackDetail(delOperatorIds, details, result, userInfo, true, operator, projectId);
            } else if (FtaOperatorTypeEnum.节点类型字段更新.getIndex().equals(operator.getOperatorType())) {
                // 执行sql，恢复节点类型和部分计算参数的属性字段
                runSql(operator.getReverseSql());
                // 将其它节点类型恢复为底事件时，验证这个底事件是否需要更新为重复事件
                checkMirrors(projectId, userInfo);
                operatorMapper.deleteById(operator.getId());
            } else if (FtaOperatorTypeEnum.转移门转为子集.getIndex().equals(operator.getOperatorType()) && StringUtils.isNotBlank(operator.getRelateId())) {
                update(userInfo, FtaNode.builder().id(operator.getRelateId()).typeCode(FtaNodeEnum.转移门.getIndex()).build(), false);
                operatorMapper.deleteById(operator.getId());
                result.put(FtaConstants.REFRESH_ANALYSIS_TREE, true);
                return ServerResponse.createBySuccess(result);
            } else if (FtaOperatorTypeEnum.子集转为转移门.getIndex().equals(operator.getOperatorType()) && StringUtils.isNotBlank(operator.getRelateId())) {
                update(userInfo, FtaNode.builder().id(operator.getRelateId()).typeCode(FtaNodeEnum.或门.getIndex()).build(), false);
                operatorMapper.deleteById(operator.getId());
                result.put(FtaConstants.REFRESH_ANALYSIS_TREE, true);
                return ServerResponse.createBySuccess(result);
            }
            // 清除分析历史计算结果
            clearCalculateHistory(userInfo, analysisId, projectId);
            return ServerResponse.createBySuccess(result);
        } else {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.最多撤销五步, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
    }

    /**
     * 通过操作记录详情，操作节点、连线、分析id
     *
     * @param details
     * @param result
     * @param userInfo
     * @param isDelete true: 将被删除的数据恢复 false：将新增的数据删除
     */
    void rollbackDetail(List<Integer> operatorIds, List<FtaOperatorDetail> details, JSONObject result, HeaderUserInfo userInfo, boolean isDelete, FtaOperator operator, String projectId) {
        ArrayList<String> nodeIds = new ArrayList<>();
        ArrayList<String> edgeIds = new ArrayList<>();
        ArrayList<String> analysisIds = new ArrayList<>();
        ArrayList<Integer> detailIds = new ArrayList<>();
        details.forEach(detail -> {
            detailIds.add(detail.getId());
            if (StringUtils.isNotBlank(detail.getNodeId())) {
                nodeIds.add(detail.getNodeId());
            } else if (StringUtils.isNotBlank(detail.getEdgeId())) {
                edgeIds.add(detail.getEdgeId());
            } else if (StringUtils.isNotBlank(detail.getAnalysisId())) {
                analysisIds.add(detail.getAnalysisId());
            }
        });
        // 整理sql
        StringBuilder sql;
        if (isDelete) {
            sql = operatorService.getUpdateSql(nodeIds, edgeIds, analysisIds);
        } else {
            if (!CollectionUtils.isEmpty(nodeIds)) {
                // 回退添加操作时，需要验证之前添加的节点是否包含了子分析，若包含则需要同时删除这些分析
                analysisIds.addAll(nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                                .select(FtaNode::getChildrenAnalysisId)
                                .eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex())
                                .in(FtaNode::getId, nodeIds)
                                .isNotNull(FtaNode::getChildrenAnalysisId))
                        .stream().map(FtaNode::getChildrenAnalysisId).collect(Collectors.toList()));
            }
            sql = operatorService.getDeleteSql(nodeIds, edgeIds, analysisIds.stream().distinct().collect(Collectors.toList()));
        }
        // 删除操作表记录的sql
        sql.append(operatorService.getDeleteSql(null, null, null, operatorIds, detailIds));
        StringBuilder builder = new StringBuilder();
        if (operator != null && StringUtils.isNotBlank(operator.getRelateId())) {
            // 撤销节点类型转换操作时，需要恢复被更新的节点类型
            builder.append("UPDATE fta_node SET ").append("type_code = '").append(operator.getReverseSql()).append("', ");
            if (operator.getEdgeOrder() != null) {
                builder.append("REQUIRED_PATH = ").append(operator.getEdgeOrder()).append(" , ");
            }
            // 去掉最后一个逗号
            int start = builder.lastIndexOf(",");
            sql.append(builder.substring(0, start));
            sql.append(builder.substring(start + 1));
            sql.append("WHERE id = '").append(operator.getRelateId()).append("';");
        }
        if (StringUtils.isNotBlank(sql)) {
            runSql(sql.toString());
        }
        checkMirrors(projectId, userInfo);
        // 是否需要前端刷新分析树
        if (!CollectionUtils.isEmpty(analysisIds)) {
            result.put(FtaConstants.REFRESH_ANALYSIS_TREE, true);
        }
    }

    /**
     * 验证当前分析集中的底事件和重复事件是否符合逻辑要求
     *
     * @param projectId
     * @param userInfo
     */
    public void checkMirrors(String projectId, HeaderUserInfo userInfo) {
        List<String> allAnalysisId = getAllAnalysisIdByProjectId(projectId, userInfo.getUserName());
        // 查找当前分析集中，是否存在masterId不为空的底事件或重复事件
        nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>()
                        .select(FtaNode::getMasterId, FtaNode::getId, FtaNode::getTypeCode)
                        .in(FtaNode::getAnalysisId, allAnalysisId)
                        .in(FtaNode::getTypeCode, FtaNodeEnum.底事件.getIndex(), FtaNodeEnum.重复事件.getIndex())
                        .isNotNull(FtaNode::getMasterId)
                        .ne(FtaNode::getMasterId, StringUtils.EMPTY))
                .stream().collect(Collectors.groupingBy(FtaNode::getMasterId))
                .forEach((masterId, list) -> {
                    if (list.size() > 1) {
                        // 如果masterId关联的底事件或重复事件的数量大于1，则需要更新这些事件为重复事件
                        // 需要更新为重复事件的底事件ids
                        List<String> updateIds = list.stream()
                                .filter(node -> FtaNodeEnum.底事件.getIndex().equals(node.getTypeCode()))
                                .map(FtaNode::getId).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(updateIds)) {
                            // 将这些底事件更新为重复事件
                            nodeMapper.update(FtaNode.builder().typeCode(FtaNodeEnum.重复事件.getIndex()).build(),
                                    new LambdaQueryWrapper<FtaNode>()
                                            .in(FtaNode::getAnalysisId, allAnalysisId)
                                            .in(FtaNode::getId, updateIds));
                        }
                    } else {
                        // 如果这个事件是重复事件，则更新这个事件为底事件
                        if (FtaNodeEnum.重复事件.getIndex().equals(list.get(0).getTypeCode())) {
                            nodeMapper.update(FtaNode.builder().typeCode(FtaNodeEnum.底事件.getIndex()).build(),
                                    new LambdaQueryWrapper<FtaNode>()
                                            .in(FtaNode::getAnalysisId, allAnalysisId)
                                            .eq(FtaNode::getId, list.get(0).getId()));
                        }
                    }
                });
    }

    /**
     * 回退剪切操作时，恢复连线顺序（此时，只能是右移）
     *
     * @param fatherMiddleId 该中间事件上方的门的的父中间事件id
     * @param middleId       需要调整顺序的中间事件的id（targetId）
     * @param order          原顺序
     */
    @Transactional(rollbackFor = Exception.class)
    public void recoverOrder(String middleId, String fatherMiddleId, Integer order) {
        FtaNode targetGate = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getFatherId, fatherMiddleId));
        nodeMapper.moveRight(targetGate.getId(), targetGate.getAnalysisId(), order);
        nodeMapper.update(FtaNode.builder().edgeOrder(order).build(), new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getFatherId, targetGate.getId())
                .eq(FtaNode::getId, middleId));
    }

    /**
     * 生成最小割集接口：根据汪博士计算程序适配
     *
     * @param userInfo
     * @param analysisId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse generateSysEquationWang(HeaderUserInfo userInfo, String analysisId) {
        //整理数据
        InputFileDataBo inputFileDataBo = assemblyDataWang(analysisId, userInfo);
        log.error("1 数据组装成功");
        HashMap<String, FtaNode> updateResultNodes = inputFileDataBo.getUpdateResultNodes();
        HashMap<String, String> gateIdAndTransferIdMap = inputFileDataBo.getGateIdAndTransferIdMap();
        FtaAnalysisCalcResult ftaAnalysisCalcResult = new FtaAnalysisCalcResult();
        // 是否全部为固定失效概率
        boolean isAllFixedProd = true;
        // 生成汪博士程序可计算的文件
        try {
            File temp = new File(allVariableConfig.getFtaCalculateDir() + "inputfiles\\temp.ftp");
            temp.createNewFile();

            FileWriter write = new FileWriter(temp);
            BufferedWriter bw = new BufferedWriter(write);
            bw.write("**CHAR32\n");
            ArrayList<FtaGateBo> tree = inputFileDataBo.getFtaTreeBos();
            // 将顶事件放到第一个来计算
            for (FtaGateBo ftaGateBo : tree) {
                StringBuilder eventIds = new StringBuilder();
                if (inputFileDataBo.getPeakEventId().equals(ftaGateBo.getOutId())) {
                    ftaGateBo.getInIds().forEach(item -> eventIds.append(item).append(" "));
                    bw.write(ftaGateBo.getOutId() + " " + (FtaNodeEnum.表决门.getUriName().equals(ftaGateBo.getGateType()) && ftaGateBo.getInIds().size() > ftaGateBo.getRequiredPath() ? ftaGateBo.getRequiredPath() : FtaNodeEnum.getMathematicsByUriName(ftaGateBo.getGateType())) + " " + eventIds + "\n");
                }
            }
            for (FtaGateBo ftaGateBo : tree) {
                StringBuilder eventIds = new StringBuilder();
                if (!inputFileDataBo.getPeakEventId().equals(ftaGateBo.getOutId())) {
                    ftaGateBo.getInIds().forEach(item -> eventIds.append(item).append(" "));
                    bw.write(ftaGateBo.getOutId() + " " + (FtaNodeEnum.表决门.getUriName().equals(ftaGateBo.getGateType()) && ftaGateBo.getInIds().size() > ftaGateBo.getRequiredPath() ? ftaGateBo.getRequiredPath() : FtaNodeEnum.getMathematicsByUriName(ftaGateBo.getGateType())) + " " + eventIds + "\n");
                }
            }
            bw.write("ENDTREE\n");
            bw.write("PROCESS" + inputFileDataBo.getProcess());
            bw.write("\nIMPORT\n");
            // 待计算的底事件
            ArrayList<FtaNode> events = inputFileDataBo.getFtaEllipseNodes();
            for (int i = 0; i < events.size(); i++) {
                if (isAllFixedProd && !FtaDistributingTypeCodeEnum.FixedProb.getIndex().equals(events.get(i).getDistributionTypeCode())) {
                    // 存在非固定失效概率的事件时，在计算完毕后只保存割集，不保存失效概率
                    isAllFixedProd = false;
                }
            }
            for (FtaNode event : events) {
                if (isAllFixedProd) {
                    bw.write("    " + event.getP1Value() + " " + event.getId() + "\n");
                } else {
                    bw.write("    0.01 " + event.getId() + "\n");
                }
            }
            bw.write("LIMIT     1.00E-099");
            bw.close();
            write.close();
            log.error("2 inputfiles生成成功");

            //删除一个文件夹下的所有文件(包括子目录内的文件)
            File file = new File(allVariableConfig.getFtaCalculateDir() + "outputfiles\\");//输入要删除文件目录的绝对路径
            if (FileUtil.deleteFile(file)) {
                log.error("3 历史计算文件删除成功");
            }

            log.error("4 开始执行计算程序");
            // 执行计算程序，并等待计算程序执行完成
            // 创建ProcessBuilder对象
            ProcessBuilder pb = new ProcessBuilder(allVariableConfig.getFtaCalculateDir() + "start.bat");
            pb.redirectErrorStream(true);//将错误流中的数据合并到输入流
            // 启动进程
            Process process = pb.start();
            // 等待进程执行完成
            while (process.isAlive()) {
                process.waitFor();
            }
        } catch (Exception e) {
            log.error("计算失败:", e);
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.计算失败请联系管理员, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
        log.error("5 计算完毕");

        String outPathName = allVariableConfig.getFtaCalculateDir() + "outputfiles\\"; // 绝对路径或相对路径都可以，写入文件时演示相对路径,读取以上路径的input.txt文件
        String[] events = inputFileDataBo.getProcess().toString().trim().split(" ");
        long millis = System.currentTimeMillis();
        // 判断计算程序是否完成（文件是否存在）
        while (FileUtil.isFile(outPathName, events[0]) && (System.currentTimeMillis() - millis) <= 2000) {
            try {
                TimeUnit.MILLISECONDS.sleep(100);//毫秒
            } catch (InterruptedException e) {
                log.error("计算失败:", e);
                throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.计算失败请联系管理员, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
            }
        }

        log.error("6 读取计算结果文件");
        // 计算程序执行完成后，读取计算结果文件（批量）
        ArrayList<String> names = new ArrayList<>();
        HashMap<String, String> calculateResultQ = new HashMap<>();
        for (String name : events) {
            if (!names.contains(name)) {
                // 开始读取计算结果
                try (FileReader reader = new FileReader(outPathName + name + ".fre");
                     BufferedReader br = new BufferedReader(reader)
                ) {
                    String eventId = null;
                    String value = null;
                    HashMap<String, String> eventIdNameMap = inputFileDataBo.getEventIdNameMap();
                    HashMap<String, String> idAndFatherMiddleIdMap = inputFileDataBo.getIdAndFatherMiddleIdMap();
                    HashMap<String, String> middleIdAndChildIdMap = inputFileDataBo.getMiddleIdAndChildIdMap();
                    StringBuilder minCutAndValue = new StringBuilder();
                    String line;
                    int no = 0;
                    boolean isCutset = false;//最小割集
                    boolean isEvent = false;//该事件中各底事件概率
                    while ((line = br.readLine()) != null) {
                        no++;
                        // 一次读入一行数据
                        if (StringUtils.isNotBlank(line)) {
                            if (line.trim().contains("CUTSET")) {
                                // 开始读取最小割集
                                isEvent = false;
                                isCutset = true;
                                continue;
                            }
                            if (line.trim().contains("EVENT")) {
                                // 读取该中间事件包含的底事件及概率
                                isEvent = true;
                                isCutset = false;
                                continue;
                            }
                            if (isCutset && no == 2) {
                                // 读取该中间事件或顶事件的名称及失效概率
                                String[] event = line.split(" {4}");
                                eventId = event[0];
                                value = event[1];
                                // 更新中间事件的Q值
                                if (updateResultNodes.get(event[0]) != null) {
                                    updateResultNodes.get(event[0]).setQ(isAllFixedProd ? event[1] : null);
                                }
                                // 更新中间事件的直接子门的Q值
                                if (updateResultNodes.get(middleIdAndChildIdMap.get(event[0])) != null) {
                                    updateResultNodes.get(middleIdAndChildIdMap.get(event[0])).setQ(isAllFixedProd ? event[1] : null);

                                    // 更新该门对应的转移门的Q
                                    String gateId = updateResultNodes.get(middleIdAndChildIdMap.get(event[0])).getId();
                                    if (gateIdAndTransferIdMap.get(gateId) != null) {
                                        updateResultNodes.put(gateIdAndTransferIdMap.get(gateId), FtaNode.builder().id(gateIdAndTransferIdMap.get(gateId)).q(isAllFixedProd ? event[1] : null).build());
                                    }
                                }
                            }
                            if (isCutset && no >= 4) {
                                // 读取盖中间事件的最小割集及失效概率
                                String[] cut = line.trim().split(" {2}");
                                String[] cutIds = cut[1].split(" ");
                                StringBuilder stringBuilder = new StringBuilder();
                                for (String cutId : cutIds) {
                                    stringBuilder.append(eventIdNameMap.get(idAndFatherMiddleIdMap.get(cutId))).append(" ");
                                }
                                minCutAndValue.append("(").append(stringBuilder).append(") ").append(isAllFixedProd ? cut[0] : StringUtils.EMPTY).append("\n");
                            }
                            if (isEvent) {
                                // 读取该中间事件包含的底事件及概率
                                String[] event = line.trim().split(" {4}");
                                if (event.length == 2) {
                                    if (StringUtils.isNotBlank(event[0]) && StringUtils.isNotBlank(event[1])) {
                                        // 读取底事件计算结果
                                        calculateResultQ.put(event[0], isAllFixedProd ? event[1] : null);
                                    }
                                }
                            }
                        }
                    }
                    // 需要更新的计算结果
                    if (updateResultNodes.get(eventId) != null) {
                        updateResultNodes.get(eventId).setCutset(minCutAndValue.toString());
                    }
                    if (inputFileDataBo.getPeakEventId().equals(eventId)) {
                        ftaAnalysisCalcResult.setSysMinCutsets("事件编号：" + eventIdNameMap.get(eventId) + "\n失效概率：" + value + "\n最小割集：\n" + minCutAndValue);
                    }
                } catch (Exception e) {
                    log.error("计算失败:", e);
                    throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.计算失败请联系管理员, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
                }

                names.add(name);
            }
        }
        log.error("7 读取计算结果文件成功");
        // 更新每个节点的计算结果（待完善，大数据）
        List<FtaNode> updateNodes = updateResultNodes.values().stream()
                .peek(node -> {
                    // 写入底事件和重复事件的计算结果
                    if (calculateResultQ.get(node.getId()) != null) {
                        node.setQ(calculateResultQ.get(node.getId()));
                    } else if (calculateResultQ.get(node.getMasterId()) != null) {
                        node.setQ(calculateResultQ.get(node.getMasterId()));
                    }
                }).collect(Collectors.toList());
        nodeMapper.updateNodesCalData(updateNodes);
        log.error("8 数据库更新成功");

        return ServerResponse.createBySuccess(ftaAnalysisCalcResult);
    }

    /**
     * 根据分析id查询最小割集
     *
     * @param ftaId
     * @return
     */
    public ServerResponse<FtaAnalysisCalcResult> getSysEquation(String ftaId) {
        FtaNode node = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getAnalysisId, ftaId).eq(FtaNode::getTypeCode, FtaNodeEnum.顶事件.getIndex()));
        if (node != null) {
            // 当前分析的顶事件存在最小割集时，使用当前数据
            if (StringUtils.isNotBlank(node.getCutset())) {
                return ServerResponse.createBySuccess(FtaAnalysisCalcResult.builder()
                        .ftaId(ftaId).sysMinCutsets(node.getCutset()).build());
            } else if (nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex()).eq(FtaNode::getChildrenAnalysisId, node.getAnalysisId())) == 1) {
                // 当前分析的存在关联的转移门时，使用关联的转移门上面的中间事件的数据
                FtaNode transfer = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex()).eq(FtaNode::getChildrenAnalysisId, node.getAnalysisId()));
                FtaNode fatherNode = nodeMapper.selectById(transfer.getFatherId());
                return ServerResponse.createBySuccess(FtaAnalysisCalcResult.builder()
                        .ftaId(ftaId).sysMinCutsets(fatherNode.getCutset()).build());
            }
        }
        return ServerResponse.createBySuccess();
    }

    /**
     * 组装接口调用所需参数
     *
     * @return
     */
    public InputFileDataBo assemblyDataWang(String analysisId, HeaderUserInfo userInfo) {
        StringBuilder errorMessage1 = new StringBuilder();
        StringBuilder errorMessage2 = new StringBuilder();
        List<FtaNode> allNodes;
        List<FtaEdge> edges;
        List<FtaNode> gates;
        // 转移门计算时：转移门对应的分析中第一个门的id 和 转移门id
        HashMap<String, String> gateIdAndTransferIdMap = new HashMap<>();
        // 判断当前分析中是否包含转移门及当前分析是否存在父分析
        if (nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>()
                .or(o -> o.eq(FtaNode::getTypeCode, FtaNodeEnum.转移门.getIndex())
                        .eq(FtaNode::getAnalysisId, analysisId))
                .or(o -> o.eq(FtaNode::getChildrenAnalysisId, analysisId))) > 0) {
            allNodes = new ArrayList<>();
            edges = new ArrayList<>();
            FtaAnalysis analysis = analysisMapper.selectById(analysisId);
            // 包含转移门的情况下，需要提前整理数据
            // 获取跟当前分析所在树中的所有的分析id
            ArrayList<String> analysisIds = new ArrayList<>();
            List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(analysis.getProjectId(), userInfo.getUserId());
            analysisIds.add(analysisId);
            FtaAnalysisService.getAnalysisIds(analysisId, analysisIds, allAnalysis);
            // 找到所有节点
            List<FtaNode> allPreNodes = nodeMapper.selectList(
                    new LambdaQueryWrapper<FtaNode>()
                            .in(FtaNode::getAnalysisId, analysisIds));
            List<FtaEdge> allPreEdges = generateEdgesByNodes(allPreNodes);
            HashMap<String, String> nodeIdAndChildrenId = new HashMap<>();
            allPreEdges.forEach(edge -> nodeIdAndChildrenId.put(edge.getSourceId(), edge.getTargetId()));
            // 需要排除的节点id
            ArrayList<String> delNodeIds = new ArrayList<>();
            // 需要排除的连线id关联的节点id
            ArrayList<String> delNodeIdsAndEdgeIds = new ArrayList<>();
            // 需要替换的节点id
            HashMap<String, String> updateNodeIds = new HashMap<>();
            HashMap<String, String> updateEdgeIds = new HashMap<>();
            allPreNodes.forEach(item -> {
                // 找到所有转移门
                if (FtaNodeEnum.转移门.getIndex().equals(item.getTypeCode())) {
                    delNodeIds.add(item.getId());
                    allPreNodes.forEach(in -> {
                        if (FtaNodeEnum.顶事件.getIndex().equals(in.getTypeCode()) && item.getChildrenAnalysisId().equals(in.getAnalysisId())) {
                            // 需要直接排除的节点id
                            delNodeIds.add(in.getId());
                            // 需要排除的线的sourceId或targetId
                            delNodeIdsAndEdgeIds.add(in.getId());
                            // 转移门id与关联的分析的第一个门id
                            updateEdgeIds.put(item.getId(), nodeIdAndChildrenId.get(in.getId()));
                            // 顶事件id与关联的转移门的父id
                            updateNodeIds.put(in.getId(), item.getFatherId());
                            // 转移门对应的分析中第一个门的id 和 转移门id
                            gateIdAndTransferIdMap.put(nodeIdAndChildrenId.get(in.getId()), item.getId());
                        }
                    });
                }
            });
            // 将包含转移门的分析数据转为一个整体
            // 排除转移门id，及转移门相关联的顶事件id
            // 更新转移门相关联的分析的第一个门的父id为转移门的父id
            allPreNodes.forEach(item -> {
                if (!delNodeIds.contains(item.getId())) {
                    if (updateNodeIds.get(item.getFatherId()) != null) {
                        item.setFatherId(updateNodeIds.get(item.getFatherId()));
                    }
                    allNodes.add(item);
                }
            });
            // 排除转移门相关联的顶事件id关联的线
            // 更新转移门上面的线的targetId为转移门关联的分析的第一个门的id
            allPreEdges.forEach(item -> {
                if (!delNodeIdsAndEdgeIds.contains(item.getSourceId()) && !delNodeIdsAndEdgeIds.contains(item.getTargetId())) {
                    if (updateEdgeIds.get(item.getTargetId()) != null) {
                        item.setTargetId(updateEdgeIds.get(item.getTargetId()));
                    }
                    edges.add(item);
                }
            });
            gates = allPreNodes.stream().filter(node -> FtaNodeEnum.与门.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.或门.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.表决门.getIndex().equals(node.getTypeCode())).collect(Collectors.toList());
        } else {
            allNodes = nodeMapper.selectList(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getAnalysisId, analysisId));
            edges = generateEdgesByNodes(allNodes);
            gates = allNodes.stream().filter(node -> FtaNodeEnum.与门.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.或门.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.表决门.getIndex().equals(node.getTypeCode())).collect(Collectors.toList());
        }

        // 顶事件id
        final String[] peakEventId = {null};
        // 汪博士计算接口所需数据：计算哪些事件（顶事件、中间事件、底事件）（目前为计算所有事件，待完善）
        final StringBuilder[] process = {new StringBuilder()};
        // 底事件集合
        ArrayList<FtaNode> ftaEllipseNodes = new ArrayList<>();
        // 重复事件
        ArrayList<String> validMirrors = new ArrayList<>();
        // 事件id与名称对应map，用于计算结果数据读取后的id名称转换
        HashMap<String, String> eventIdNameMap = new HashMap<>();
        // 中间事件和顶事件的id与信息map，用于计算完成之后更新计算结果（待完善）
        HashMap<String, FtaNode> updateResultNodes = new HashMap<>();
        // 重复事件id与对应事件id的Map
        HashMap<String, String> idAndMasterIdMap = new HashMap<>();
        // 底事件、重复事件、门的id与父级的中间事件id的Map
        HashMap<String, String> idAndFatherMiddleIdMap = new HashMap<>();
        // 中间事件id与底事件、重复事件、门的id的Map
        HashMap<String, String> middleIdAndChildIdMap = new HashMap<>();
        // 所有门的id
        List<String> allGateIds = gates.stream().map(FtaNode::getId).collect(Collectors.toList());
        allNodes.forEach(item -> {
            if (FtaNodeEnum.顶事件.getIndex().equals(item.getTypeCode())) {
                peakEventId[0] = item.getId();
            }
            if (FtaNodeEnum.中间事件.getIndex().equals(item.getTypeCode()) ||
                    FtaNodeEnum.顶事件.getIndex().equals(item.getTypeCode())) {
                updateResultNodes.put(item.getId(), item);
                eventIdNameMap.put(item.getId(), item.getCode());
            }
            if (FtaNodeEnum.底事件.getIndex().equals(item.getTypeCode())
                    || FtaNodeEnum.未展开事件.getIndex().equals(item.getTypeCode())) {
                ftaEllipseNodes.add(item);
                eventIdNameMap.put(item.getId(), item.getName());
                updateResultNodes.put(item.getId(), item);
            }
            if (FtaNodeEnum.重复事件.getIndex().equals(item.getTypeCode())) {
                if (!validMirrors.contains(item.getMasterId())) {
                    ftaEllipseNodes.add(FtaNode.builder()
                            .id(item.getMasterId()).distributionTypeCode(item.getDistributionTypeCode())
                            .p1Value(item.getP1Value()).p2Value(item.getP2Value()).p3Value(item.getP3Value())
                            .build());
                    validMirrors.add(item.getMasterId());
                }
                idAndMasterIdMap.put(item.getId(), item.getMasterId());
                eventIdNameMap.put(item.getId(), item.getName());
                updateResultNodes.put(item.getId(), item);
                idAndFatherMiddleIdMap.put(item.getMasterId(), item.getFatherId());
            }
            if (FtaNodeEnum.与门.getIndex().equals(item.getTypeCode())
                    || FtaNodeEnum.或门.getIndex().equals(item.getTypeCode())
                    || FtaNodeEnum.表决门.getIndex().equals(item.getTypeCode())) {
                updateResultNodes.put(item.getId(), item);
            }
        });

        ArrayList<FtaGateBo> ftaGateBos = new ArrayList<>();

        if (gates.size() > allVariableConfig.getGateMaxNumber()) {
            throw new DescribeException(String.format(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.测试环境暂不支持超过n个门的分析计算, userInfo.getLanguageKey()), allVariableConfig.getGateMaxNumber()), ResponseStatus.WARN.getStatus());
        }
        gates.forEach(gate -> {
            String outId = "-1"; // 向上↑的出线
            //门节点
            ArrayList<String> inIds = new ArrayList<>();
            for (FtaEdge edge : edges) {
                if (gate.getId().equals(edge.getTargetId())) {
                    // 找到该门上面的 中间事件或顶事件
                    outId = edge.getSourceId();
                    if (peakEventId[0].equals(outId)) {
                        process[0] = new StringBuilder(" ").append(outId).append(process[0]);
                    } else {
                        process[0].append(" ").append(outId);
                    }
                    middleIdAndChildIdMap.put(outId, gate.getId());
                } else if (gate.getId().equals(edge.getSourceId())) {
                    // 找到该门下面的 中间事件
                    String inId = edge.getTargetId();
                    // 再找到该中间事件的直接输入：底事件、重复事件、门
                    // 底事件和重复事件需要直接添加id，如果是门则需要添加上一级的中间事件id
                    for (FtaEdge edgeIns : edges) {
                        if (inId.equals(edgeIns.getSourceId())) {
                            idAndFatherMiddleIdMap.put(edgeIns.getTargetId(), inId);
                            if (allGateIds.contains(edgeIns.getTargetId())) {
                                // 门
                                inIds.add(inId);
                            } else if (idAndMasterIdMap.get(edgeIns.getTargetId()) != null) {
                                // 如果此id是重复事件，则使用该重复时间的对应事件id参与计算
                                inIds.add(idAndMasterIdMap.get(edgeIns.getTargetId()));
                            } else {
                                // 底事件
                                inIds.add(edgeIns.getTargetId());
                            }
                            break;
                        }
                    }
                }
            }
            if (inIds.isEmpty()) {
                FtaNode fatherNode = nodeMapper.selectById(gate.getFatherId());
                errorMessage1.append(fatherNode.getCode()).append(" ");
            } else if (FtaNodeEnum.表决门.getIndex().equals(gate.getTypeCode()) && (gate.getRequiredPath() <= 0 || gate.getRequiredPath() > inIds.size())) {
                // 必要路线必须大于0且小于等于入线数量
                FtaNode fatherNode = nodeMapper.selectById(gate.getFatherId());
                errorMessage2.append(fatherNode.getCode()).append(" ");
            }
            FtaGateBo ftaGateBo = FtaGateBo.builder().gateId(gate.getId())
                    .gateName(gate.getName())
                    .gateType(FtaNodeEnum.getUriNameByIndex(gate.getTypeCode()))
                    .requiredPath(gate.getRequiredPath())
                    .inIds(inIds)
                    .outId(outId).build();
            ftaGateBos.add(ftaGateBo);
        });
        if (StringUtils.isNotBlank(errorMessage1)) {
            throw new DescribeException(String.format(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请检查gates下是否存在事件, userInfo.getLanguageKey()), errorMessage1), ResponseStatus.WARN.getStatus());
        } else if (StringUtils.isNotBlank(errorMessage2)) {
            throw new DescribeException(String.format(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请检查表决门gates下的事件数量是否符合数量要求, userInfo.getLanguageKey()), errorMessage2), ResponseStatus.WARN.getStatus());
        }
        return InputFileDataBo.builder().ftaTreeBos(ftaGateBos)
                .ftaEllipseNodes(ftaEllipseNodes).process(process[0]).peakEventId((peakEventId[0]))
                .updateResultNodes(updateResultNodes).eventIdNameMap(eventIdNameMap)
                .idAndFatherMiddleIdMap(idAndFatherMiddleIdMap)
                .middleIdAndChildIdMap(middleIdAndChildIdMap)
                .gateIdAndTransferIdMap(gateIdAndTransferIdMap).build();
    }

    /**
     * 绘图接口，暂时只支持不包含固定失效概率的分析绘图
     *
     * @return
     */
    public ServerResponse image(HeaderUserInfo userInfo, FtaImageVo imageVo) {
        JSONObject result = new JSONObject();
        //节点数据
        List<String> mirrorIds = new ArrayList<>();
        List<FtaNode> nodeList = nodeMapper.selectIdAndFatherName(imageVo.getAnalysisId())
                .stream().filter(node -> {
                    if (FtaNodeEnum.重复事件.getIndex().equals(node.getTypeCode())) {
                        if (mirrorIds.contains(node.getMasterId())) {
                            return false;
                        } else {
                            mirrorIds.add(node.getMasterId());
                            return true;
                        }
                    } else {
                        return true;
                    }
                }).collect(Collectors.toList());

        // 节点绘图参数
        String moreParameter = ObjectUtils.isEmpty(imageVo.getXaxisStart()) || ObjectUtils.isEmpty(imageVo.getXaxisEnd()) ? "?autoscaleX=true&xmin=0&xmax=0&NumofPoints=200" : "?autoscaleX=false&xmin=" + imageVo.getXaxisStart() + "&xmax=" + imageVo.getXaxisEnd() + "&NumofPoints=200";

        if (CollectionUtils.isEmpty(imageVo.getCheckedNodes())) {
            // 第一次进入绘图页面，只画第一个点
            FtaNode first = nodeList.get(0);
            nodeList.clear();
            nodeList.add(first);
        } else {
            nodeList = nodeList.stream().filter(node -> {
                boolean valid = false;
                for (int i = 0; i < imageVo.getCheckedNodes().size(); i++) {
                    if (!valid && imageVo.getCheckedNodes().get(i).equals(node.getId())) {
                        valid = true;
                    }
                }
                return valid;
            }).collect(Collectors.toList());
        }
        ArrayList<JSONObject> nodes = new ArrayList<>();
        for (FtaNode node : nodeList) {
            // 获取完整的画图请求url
            String apiPlotUrl = ftaRgaApiService.getApiPlotUrl(node.getDistributionTypeCode(), imageVo.getPlotType(), moreParameter);
            //整理数据
            JSONObject requestPlotData = new JSONObject();
            JSONObject data = new JSONObject();
            data.put("failureRank", new double[0]);
            requestPlotData.put("data", data);
            JSONObject results = new JSONObject();
            if (FtaDistributingTypeCodeEnum.Weibull2.getIndex().equals(node.getDistributionTypeCode())) {
                results.put("beta", node.getP1Value());
                results.put("eta", node.getP2Value());
            } else if (FtaDistributingTypeCodeEnum.Weibull3.getIndex().equals(node.getDistributionTypeCode())) {
                results.put("beta", node.getP1Value());
                results.put("eta", node.getP2Value());
                results.put("gamma", node.getP3Value());
            } else if (FtaDistributingTypeCodeEnum.Exponential.getIndex().equals(node.getDistributionTypeCode())) {
                results.put("MTTF", node.getP1Value());
            } else if (FtaDistributingTypeCodeEnum.Lognormal.getIndex().equals(node.getDistributionTypeCode())) {
                results.put("logMu", node.getP1Value());
                results.put("logSigma", node.getP2Value());
            } else if (FtaDistributingTypeCodeEnum.Normal.getIndex().equals(node.getDistributionTypeCode())) {
                results.put("mu", node.getP1Value());
                results.put("sigma", node.getP2Value());
//            } else if (FtaDistributingTypeCodeEnum.FixedProb.getIndex().equals(node.getDistributionTypeCode())) {
//                results.put("R", node.getP1Value());
                // 包含固定失效概率的数据不能计算
            }
            requestPlotData.put("results", results);
            requestPlotData.put("settings", new JSONObject());
            // 获取画图数据
            String nodePlot = ftaRgaApiService.getResultsCallApi(requestPlotData.toString(), apiPlotUrl, EwbConstants.RGA_API_CONTENTTYPE_1, EwbConstants.RGA_API_ACCEPTTYPE_1);
            JSONObject nodePlotJsonObject = JSONObject.parseObject(nodePlot);
            if (nodePlotJsonObject == null) {
                return ServerResponse.createByErrorMessage(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.计算失败请检查数据或设置, userInfo.getLanguageKey()));
            }
            nodePlotJsonObject.put("nodeId", node.getId());
            nodePlotJsonObject.put("nodeName", node.getCode());
            nodes.add(nodePlotJsonObject);
        }
        result.put("nodeLineData", nodes);
        return ServerResponse.createBySuccess(result);
    }

    /**
     * 项目启动时，清洗数据
     */
    public void init() {
        log.warn("FTA 启动！");
    }

    /**
     * 更新节点展开收缩
     *
     * @param nodeId
     * @param leaf
     * @return
     */
    public ServerResponse<String> updateLeaf(String nodeId, Boolean leaf) {
        FtaNode node = nodeMapper.selectById(nodeId);
        if (node != null && (FtaNodeEnum.中间事件.getIndex().equals(node.getTypeCode()) || FtaNodeEnum.顶事件.getIndex().equals(node.getTypeCode()))) {
            nodeMapper.updateById(FtaNode.builder().id(nodeId).leaf(leaf).build());
        }
        return ServerResponse.createBySuccess();
    }

    /**
     * 更新连线顺序，将moveSourceId移动至moveTargetId 之前或之后的位置
     *
     * @param movedSourceEdgeTargetId
     * @param movedTargetEdgeTargetId
     * @param moveDirection           移至目标节点之前或之后
     * @return
     */
    @Transactional
    public ServerResponse<String> updateOrder(String movedSourceEdgeTargetId, String movedTargetEdgeTargetId, Integer moveDirection, HeaderUserInfo userInfo) {
        FtaNode moveSource = nodeMapper.selectById(movedSourceEdgeTargetId);
        FtaNode moveTarget = nodeMapper.selectById(movedTargetEdgeTargetId);
        if (moveSource != null && moveTarget != null) {
            String analysisId = moveSource.getAnalysisId();
            String sourceId = moveSource.getFatherId();
            if (FtaNodeMoveEnum.移至目标节点之前.getIndex().equals(moveDirection) && moveTarget.getEdgeOrder() != moveSource.getEdgeOrder() + 1) {
                // 最后一步操作为左移：将source移动到了target之前
                if (moveSource.getEdgeOrder() > moveTarget.getEdgeOrder()) {
                    // 实际左移：将一些节点右移
                    nodeMapper.moveToRight(analysisId, sourceId, moveSource.getEdgeOrder(), moveTarget.getEdgeOrder());
                    nodeMapper.updateById(FtaNode.builder().id(moveSource.getId()).edgeOrder(moveTarget.getEdgeOrder()).build());
                    // 记录回退sql
                    operatorService.moveEdge(userInfo, "UPDATE fta_node SET EDGE_ORDER = EDGE_ORDER - 1 WHERE FATHER_ID = '" + sourceId +
                            "' AND EDGE_ORDER > " + moveTarget.getEdgeOrder() +
                            " AND EDGE_ORDER <= " + moveSource.getEdgeOrder() + " AND del_flag = '0';" +
                            "UPDATE fta_node SET EDGE_ORDER = " + moveSource.getEdgeOrder() +
                            " WHERE ID = '" + moveSource.getId() + "';", analysisId);
                } else if (moveSource.getEdgeOrder() < moveTarget.getEdgeOrder()) {
                    // 实际右移：将一些节点左移
                    nodeMapper.moveToLeft(analysisId, sourceId, moveSource.getEdgeOrder(), moveTarget.getEdgeOrder() - 1);
                    nodeMapper.updateById(FtaNode.builder().id(moveSource.getId()).edgeOrder(moveTarget.getEdgeOrder() - 1).build());
                    operatorService.moveEdge(userInfo, "UPDATE fta_node SET EDGE_ORDER = EDGE_ORDER + 1 WHERE FATHER_ID = '" + sourceId +
                            "' AND EDGE_ORDER >= " + moveSource.getEdgeOrder() +
                            " AND EDGE_ORDER < " + moveTarget.getEdgeOrder() + " AND del_flag = '0';" +
                            "UPDATE fta_node SET EDGE_ORDER = " + moveSource.getEdgeOrder() +
                            " WHERE ID = '" + moveSource.getId() + "';", analysisId);
                }
            } else if (FtaNodeMoveEnum.移至目标节点之后.getIndex().equals(moveDirection) && moveTarget.getEdgeOrder() + 1 != moveSource.getEdgeOrder()) {
                // 右移：将source移动到了target之后
                if (moveSource.getEdgeOrder() < moveTarget.getEdgeOrder()) {
                    // 实际右移：将一些节点左移
                    nodeMapper.moveToLeft(analysisId, sourceId, moveSource.getEdgeOrder(), moveTarget.getEdgeOrder());
                    nodeMapper.updateById(FtaNode.builder().id(moveSource.getId()).edgeOrder(moveTarget.getEdgeOrder()).build());
                    operatorService.moveEdge(userInfo, "UPDATE fta_node SET EDGE_ORDER = EDGE_ORDER + 1 WHERE FATHER_ID = '" + sourceId +
                            "' AND EDGE_ORDER >= " + moveSource.getEdgeOrder() +
                            " AND EDGE_ORDER < " + moveTarget.getEdgeOrder() + " AND del_flag = '0';" +
                            "UPDATE fta_node SET EDGE_ORDER = " + moveSource.getEdgeOrder() +
                            " WHERE ID = '" + moveSource.getId() + "';", analysisId);
                } else if (moveSource.getEdgeOrder() > moveTarget.getEdgeOrder()) {
                    // 实际左移：将一些节点右移
                    nodeMapper.moveToRight(analysisId, sourceId, moveSource.getEdgeOrder(), moveTarget.getEdgeOrder() + 1);
                    nodeMapper.updateById(FtaNode.builder().id(moveSource.getId()).edgeOrder(moveTarget.getEdgeOrder() + 1).build());
                    operatorService.moveEdge(userInfo, "UPDATE fta_node SET EDGE_ORDER = EDGE_ORDER - 1 WHERE FATHER_ID = '" + sourceId +
                            "' AND EDGE_ORDER > " + moveTarget.getEdgeOrder() +
                            " AND EDGE_ORDER <= " + moveSource.getEdgeOrder() + " AND del_flag = '0';" +
                            "UPDATE fta_node SET EDGE_ORDER = " + moveSource.getEdgeOrder() +
                            " WHERE ID = '" + moveSource.getId() + "';", analysisId);
                }
            }
        }
        return ServerResponse.createBySuccess();
    }

    public void runSql(String sql) {
        String[] sqls = sql.split(";");
        for (int i = 0; i < sqls.length; i++) {
            if (sqls[i].contains("DELETE")) {
                com.baomidou.mybatisplus.extension.toolkit.SqlRunner.db().delete(sqls[i]);
            } else if (sqls[i].contains("UPDATE")) {
                com.baomidou.mybatisplus.extension.toolkit.SqlRunner.db().update(sqls[i]);
            }
        }
    }

    /**
     * 验证分析是否满足绘图条件，如果满足，则返回可画图的节点数据
     *
     * @param ftaId
     * @param userInfo
     * @return
     */
    public ServerResponse validPlot(String ftaId, HeaderUserInfo userInfo) {
        // 获取该分析关联的所有分析
        ArrayList<String> analysisIds = new ArrayList<>();
        FtaAnalysis analysis = analysisMapper.selectById(ftaId);
        List<FtaAnalysis> allAnalysis = getAllAnalysisByProjectId(analysis.getProjectId(), userInfo.getUserName());
        FtaAnalysisService.getAnalysisIds(ftaId, analysisIds, allAnalysis);
        // 查询这些分析中是否包含固定失效概率的事件
        if (nodeMapper.selectCount(new LambdaUpdateWrapper<FtaNode>()
                .in(FtaNode::getAnalysisId, analysisIds)
                .eq(FtaNode::getDistributionTypeCode, FtaDistributingTypeCodeEnum.FixedProb.getIndex())
                .in(FtaNode::getTypeCode, FtaNodeEnum.底事件.getIndex(),
                        FtaNodeEnum.重复事件.getIndex(), FtaNodeEnum.未展开事件.getIndex())) == 0) {
            List<String> mirrorIds = new ArrayList<>();
            List<FtaNode> nodeList = nodeMapper.selectIdAndFatherName(ftaId)
                    .stream().filter(node -> {
                        if (FtaNodeEnum.重复事件.getIndex().equals(node.getTypeCode())) {
                            if (mirrorIds.contains(node.getMasterId())) {
                                return false;
                            } else {
                                mirrorIds.add(node.getMasterId());
                                return true;
                            }
                        } else {
                            return true;
                        }
                    }).collect(Collectors.toList());
            return ServerResponse.createBySuccess(nodeList);
        }
        return ServerResponse.createBySuccess();
    }

    /**
     * 复制为同源转移门
     *
     * @param userInfo
     * @param sourceId
     * @param targetId
     * @return
     */
    public ServerResponse nodeCopyTransfer(HeaderUserInfo userInfo, String sourceId, String targetId) {
        Integer operatorType = FtaOperatorTypeEnum.复制为重复转移门.getIndex();
        FtaNode middleNode = nodeMapper.selectById(sourceId);
        FtaNode targetGate = nodeMapper.selectOne(new LambdaQueryWrapper<FtaNode>()
                .eq(FtaNode::getAnalysisId, middleNode.getAnalysisId())
                .eq(FtaNode::getFatherId, targetId));
        // 被复制的只能是中间事件或顶事件，只能复制到基本门下
        if (StringUtils.isBlank(middleNode.getAnalysisId()) || targetGate == null
                || !(FtaNodeEnum.中间事件.getIndex().equals(middleNode.getTypeCode())
                || FtaNodeEnum.顶事件.getIndex().equals(middleNode.getTypeCode()))
                || !(FtaNodeEnum.与门.getIndex().equals(targetGate.getTypeCode())
                || FtaNodeEnum.或门.getIndex().equals(targetGate.getTypeCode())
                || FtaNodeEnum.表决门.getIndex().equals(targetGate.getTypeCode()))) {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.请先选择门上方的名称框再进行粘贴, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        } else if (!middleNode.getAnalysisId().equals(targetGate.getAnalysisId())) {
            throw new DescribeException(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.不支持跨分析粘贴, userInfo.getLanguageKey()), ResponseStatus.WARN.getStatus());
        }
        // 转移门/中间事件粘贴为同源转移门
        if (!middleNode.getTypeCode().equals(FtaNodeEnum.转移门.getIndex())) {
            // 调用转为转移门方法

        }
        middleNode.setFatherId(targetGate.getId());
        middleNode.setEdgeOrder(Math.toIntExact(nodeMapper.selectCount(new LambdaQueryWrapper<FtaNode>().eq(FtaNode::getFatherId, targetGate.getId())) + 1));
        middleNode.setId(UUIDUtil.getUuid());
        nodeMapper.insert(middleNode);
        List<String> delNodeIds = new ArrayList<>();
        delNodeIds.add(middleNode.getId());
        operatorService.insert(userInfo, delNodeIds, new ArrayList<>(), middleNode.getAnalysisId(), new ArrayList<>(), operatorType);
        return ServerResponse.createBySuccess();
    }
}
