package com.nationrel.modules.fta.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.nationrel.modules.fta.entity.FtaNode;
import com.nationrel.modules.fta.entity.FtaOperator;
import com.nationrel.modules.fta.entity.FtaOperatorDetail;
import com.nationrel.modules.fta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaOperatorTypeEnum;
import com.nationrel.modules.fta.mapper.FtaOperatorDetailMapper;
import com.nationrel.modules.fta.mapper.FtaOperatorMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zhangyuanming
 * @description 针对表【fta_operator】的数据库操作Service实现
 * @createDate 2024-01-12 10:25:54
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FtaOperatorService {

    private final FtaOperatorMapper operatorMapper;
    private final FtaOperatorDetailMapper detailMapper;
    private final FtaOperatorDetailService operatorDetailService;
    private final DataSource dataSource;

    /**
     * 只能记录并回退五步操作，删除最早的那一步操作，并删除这个操作的关联操作，并删除关联操作中存储的已被逻辑删除的业务数据
     *
     * @param analysisId
     */
    public void operatorValid(String analysisId, HeaderUserInfo userInfo) {
        if (operatorMapper.selectCount(new LambdaQueryWrapper<FtaOperator>()
                .select(FtaOperator::getId)
                .eq(FtaOperator::getAnalysisId, analysisId)
                .isNull(FtaOperator::getFatherId)) >= 5) {
            ArrayList<String> delNodeIds = new ArrayList<>();
            ArrayList<String> delEdgeIds = new ArrayList<>();
            ArrayList<String> delAnalysisIds = new ArrayList<>();
            ArrayList<Integer> delOperatorIds = new ArrayList<>();
            ArrayList<Integer> delDetailIds = new ArrayList<>();
            // 删除该分析最早的操作，及关联的操作
            FtaOperator operator = operatorMapper.selectOne(new LambdaQueryWrapper<FtaOperator>()
                    .select(FtaOperator::getId, FtaOperator::getOperatorType)
                    .eq(FtaOperator::getAnalysisId, analysisId)
                    .isNull(FtaOperator::getFatherId)
                    .orderByAsc(FtaOperator::getInsertTime).last(" LIMIT 1"));
            delOperatorIds.add(operator.getId());
            // 最早的操作的关联子操作
            delOperatorIds.addAll(operatorMapper.selectList(new LambdaQueryWrapper<FtaOperator>()
                            .select(FtaOperator::getId)
                            .eq(FtaOperator::getFatherId, operator.getId()))
                    .stream().map(FtaOperator::getId).collect(Collectors.toList()));
            List<FtaOperatorDetail> details = detailMapper.selectList(new LambdaQueryWrapper<FtaOperatorDetail>()
                    .in(FtaOperatorDetail::getOperatorId, delOperatorIds));
            if (!CollectionUtils.isEmpty(details)) {
                // 删除这些操作的同时，需要删除关联的已经被逻辑删除的数据
                details.forEach(detail -> {
                    delDetailIds.add(detail.getId());
                    if (StringUtils.isNotBlank(detail.getNodeId())) {
                        delNodeIds.add(detail.getNodeId());
                    } else if (StringUtils.isNotBlank(detail.getEdgeId())) {
                        delEdgeIds.add(detail.getEdgeId());
                    } else if (StringUtils.isNotBlank(detail.getAnalysisId())) {
                        delAnalysisIds.add(detail.getAnalysisId());
                    }
                });
            }
            // 删除不必要的历史数据
            String deleteSql = getDeleteSql(delNodeIds, delEdgeIds, delAnalysisIds, delOperatorIds, delDetailIds);
            if (StringUtils.isNotBlank(deleteSql)) {
                runSql(deleteSql);
            }
        }
    }

    /**
     * 添加节点和连线时，记录删除语句
     *
     * @param userInfo
     * @param delNodeIds
     * @param delEdgeIds
     * @param analysisId     当前分析id
     * @param newAnalysisIds 添加转移门时，新增的分析id
     */
    public void insert(HeaderUserInfo userInfo, List<String> delNodeIds, List<String> delEdgeIds, String analysisId, List<String> newAnalysisIds, Integer operatorType) {
        operatorValid(analysisId, userInfo);
        StringBuilder reverseSql = new StringBuilder();
        FtaOperator operator = FtaOperator.builder()
                .analysisId(analysisId)
                .operatorType(operatorType)
                .reverseSql(reverseSql.toString())
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .insertUserName(userInfo.getUserName()).build();
        operatorMapper.insert(operator);
        ArrayList<FtaOperatorDetail> operatorDetailInserts = new ArrayList<>();
        if (!CollectionUtils.isEmpty(delNodeIds)) {
            delNodeIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).nodeId(id).build()));
        }
        if (!CollectionUtils.isEmpty(delEdgeIds)) {
            delEdgeIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).edgeId(id).build()));
        }
        if (!CollectionUtils.isEmpty(newAnalysisIds)) {
            newAnalysisIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).analysisId(id).build()));
        }
        operatorDetailService.saveBatch(operatorDetailInserts);
    }

    /**
     * 删除节点和连线时，这些数据将被逻辑删除。回退时，更新这些数据
     *
     * @param userInfo
     * @param updateNodeIds
     * @param updateEdgeIds
     * @param analysisId        当前分析id
     * @param analysisIds       需要关联删除的分析id（被删除的子集中包含转移门时）
     * @param needOperatorValid 是否为操作表的主操作：主操作需要验证当前操作步长
     */
    public void delete(Integer operatorId, HeaderUserInfo userInfo, List<String> updateNodeIds, List<String> updateEdgeIds, String nodeId, Integer order, String analysisId, ArrayList<String> analysisIds, boolean needOperatorValid) {
        if (needOperatorValid) {
            operatorValid(analysisId, userInfo);
        }
        FtaOperator operator = FtaOperator.builder()
                .analysisId(analysisId)
                .fatherId(operatorId)
                .operatorType(FtaOperatorTypeEnum.删除节点和连线.getIndex())
                .relateId(nodeId)
                .edgeOrder(order)
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .insertUserName(userInfo.getUserName()).build();
        operatorMapper.insert(operator);
        ArrayList<FtaOperatorDetail> operatorDetailInserts = new ArrayList<>();
        if (!CollectionUtils.isEmpty(updateNodeIds)) {
            updateNodeIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).nodeId(id).build()));
        }
        if (!CollectionUtils.isEmpty(updateEdgeIds)) {
            updateEdgeIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).edgeId(id).build()));
        }
        if (!CollectionUtils.isEmpty(analysisIds)) {
            analysisIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).analysisId(id).build()));
        }
        operatorDetailService.saveBatch(operatorDetailInserts);
    }

    /**
     * 记录与门、或门、表决门转为底事件的操作（当门下有子集时调用此方法）
     *
     * @param userInfo
     * @param delNodeIds
     * @param delEdgeIds
     * @param analysisId  当前分析id
     * @param analysisIds 需要关联删除的分析id（被删除的子集中包含转移门时）
     */
    public void gateToCircle(Integer operatorId, HeaderUserInfo userInfo, List<String> delNodeIds, List<String> delEdgeIds, FtaNode node, String analysisId, ArrayList<String> analysisIds) {
        operatorValid(analysisId, userInfo);
        FtaOperator operator = FtaOperator.builder()
                .analysisId(analysisId)
                .fatherId(operatorId)
                .relateId(node.getId()) // 记录被转换的节点id
                .edgeOrder(node.getRequiredPath()) // 记录被转换的节点的必要路径
                .reverseSql(node.getTypeCode()) // 记录被转换的节点的类型
                .operatorType(FtaOperatorTypeEnum.基本门转为底事件.getIndex())
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .insertUserName(userInfo.getUserName()).build();
        operatorMapper.insert(operator);
        ArrayList<FtaOperatorDetail> operatorDetailInserts = new ArrayList<>();
        if (!CollectionUtils.isEmpty(delNodeIds)) {
            delNodeIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).nodeId(id).build()));
        }
        if (!CollectionUtils.isEmpty(delEdgeIds)) {
            delEdgeIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).edgeId(id).build()));
        }
        if (!CollectionUtils.isEmpty(analysisIds)) {
            analysisIds.forEach(id -> operatorDetailInserts.add(FtaOperatorDetail.builder()
                    .operatorId(operator.getId()).analysisId(id).build()));
        }
        operatorDetailService.saveBatch(operatorDetailInserts);
    }

    /**
     * 简单类型转换（基本门互转；底事件转为基本门；底事件与未展开事件互转）
     * 这些操作仅需更新一个节点类型字段和该节点的部分属性
     *
     * @param userInfo
     * @param node       被转换的节点
     * @param analysisId 当前分析id
     * @return
     */
    public void updateNodeTypeCode(HeaderUserInfo userInfo, FtaNode node, String analysisId, Integer operatorId) {
        operatorValid(analysisId, userInfo);
        // 记录简单类型转换的更新语句，用于回退操作
        StringBuilder reverseSql = new StringBuilder("UPDATE fta_node SET ").append("type_code = '").append(node.getTypeCode()).append("', ");
        if (StringUtils.isNotBlank(node.getDistributionTypeCode())) {
            reverseSql.append("DISTRIBUTION_TYPE_CODE = '").append(node.getDistributionTypeCode()).append("', ");
        }
        if (node.getP1Value() != null) {
            reverseSql.append("P1_VALUE = '").append(node.getP1Value()).append("', ");
        }
        if (node.getP2Value() != null) {
            reverseSql.append("P2_VALUE = '").append(node.getP2Value()).append("', ");
        }
        if (node.getP3Value() != null) {
            reverseSql.append("P3_VALUE = '").append(node.getP3Value()).append("', ");
        }
        if (node.getRequiredPath() != null) {
            reverseSql.append("REQUIRED_PATH = '").append(node.getRequiredPath()).append("', ");
        }
        // 去掉最后一个逗号
        int start = reverseSql.lastIndexOf(",");
        StringBuilder builder = new StringBuilder();
        builder.append(reverseSql.substring(0, start));
        builder.append(reverseSql.substring(start + 1));
        builder.append("WHERE id = '").append(node.getId()).append("';");
        log.error(builder.toString());
        FtaOperator operator = FtaOperator.builder()
                .analysisId(analysisId)
                .fatherId(operatorId)
                .operatorType(FtaOperatorTypeEnum.节点类型字段更新.getIndex())
                .reverseSql(builder.toString())
                .relateId(node.getId())
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .insertUserName(userInfo.getUserName()).build();
        operatorMapper.insert(operator);
    }

    /**
     * 记录连线顺序调整的回退
     *
     * @param userInfo
     * @param reverseSql 通过执行sql回退
     * @param analysisId
     */
    public void moveEdge(HeaderUserInfo userInfo, String reverseSql, String analysisId) {
        operatorValid(analysisId, userInfo);
        FtaOperator operator = FtaOperator.builder()
                .analysisId(analysisId)
                .operatorType(FtaOperatorTypeEnum.更新连线顺序.getIndex())
                .reverseSql(reverseSql)
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .insertUserName(userInfo.getUserName()).build();
        operatorMapper.insert(operator);
    }

    /**
     * 记录剪切的回退
     *
     * @param userInfo
     * @param jsStr      FtaNode 对象json字符串
     * @param analysisId
     */
    public void cutNodes(HeaderUserInfo userInfo, String jsStr, String analysisId) {
        operatorValid(analysisId, userInfo);
        FtaOperator operator = FtaOperator.builder()
                .analysisId(analysisId)
                .operatorType(FtaOperatorTypeEnum.剪切.getIndex())
                .reverseSql(jsStr)
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .insertUserName(userInfo.getUserName()).build();
        operatorMapper.insert(operator);
    }

    /**
     * 记录子集转为转移门的操作
     *
     * @param nodeId
     * @param analysisId
     * @param userInfo
     */
    public void toTransferGate(String nodeId, String analysisId, HeaderUserInfo userInfo) {
        if (StringUtils.isNotBlank(nodeId)) {
            operatorValid(analysisId, userInfo);
            operatorMapper.insert(FtaOperator.builder()
                    .analysisId(analysisId)
                    .operatorType(FtaOperatorTypeEnum.子集转为转移门.getIndex())
                    .relateId(nodeId)
                    .insertTime(new Date())
                    .insertUserId(userInfo.getUserId())
                    .insertUserName(userInfo.getUserName()).build());
        }
    }

    /**
     * 记录转移门转为子集的操作
     *
     * @param nodeId
     * @param analysisId
     * @param userInfo
     */
    public void toSubset(String nodeId, String analysisId, HeaderUserInfo userInfo) {
        if (StringUtils.isNotBlank(nodeId)) {
            operatorValid(analysisId, userInfo);
            operatorMapper.insert(FtaOperator.builder()
                    .analysisId(analysisId)
                    .operatorType(FtaOperatorTypeEnum.转移门转为子集.getIndex())
                    .relateId(nodeId)
                    .insertTime(new Date())
                    .insertUserId(userInfo.getUserId())
                    .insertUserName(userInfo.getUserName()).build());
        }
    }

    private StringBuilder getInSql(List<String> ids) {
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < ids.size() - 1; i++) {
            stringBuilder.append(ids.get(i)).append("','");
        }
        stringBuilder.append(ids.get(ids.size() - 1));
        return stringBuilder;
    }

    private StringBuilder getInIntegerSql(List<Integer> ids) {
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < ids.size() - 1; i++) {
            stringBuilder.append(ids.get(i)).append(",");
        }
        stringBuilder.append(ids.get(ids.size() - 1));
        return stringBuilder;
    }

    /**
     * 获取执行恢复的sql
     *
     * @param nodeIds
     * @param edgeIds
     * @param analysisIds
     * @return
     */
    public StringBuilder getUpdateSql(List<String> nodeIds, List<String> edgeIds, List<String> analysisIds) {
        StringBuilder reverseSql = new StringBuilder();
        if (!CollectionUtils.isEmpty(nodeIds)) {
            reverseSql.append(new StringBuilder("UPDATE fta_node SET del_flag='0' WHERE id IN('").append(getInSql(nodeIds)).append("');"));
        }
        if (!CollectionUtils.isEmpty(edgeIds)) {
            reverseSql.append(new StringBuilder("UPDATE fta_edge SET del_flag='0' WHERE id IN('").append(getInSql(edgeIds)).append("');"));
        }
        if (!CollectionUtils.isEmpty(analysisIds)) {
            reverseSql.append(new StringBuilder("UPDATE fta_analysis SET del_flag='0' WHERE analysis_id IN('").append(getInSql(analysisIds)).append("');"));
        }
        return reverseSql;
    }

    /**
     * 获取执行删除的sql，无论数据是否被逻辑删除，都直接删掉
     *
     * @param nodeIds
     * @param edgeIds
     * @param analysisIds
     * @return
     */
    public StringBuilder getDeleteSql(List<String> nodeIds, List<String> edgeIds, List<String> analysisIds) {
        StringBuilder reverseSql = new StringBuilder();
        if (!CollectionUtils.isEmpty(nodeIds)) {
            reverseSql.append(new StringBuilder("DELETE FROM fta_node WHERE id IN('").append(getInSql(nodeIds)).append("');"));
        }
        if (!CollectionUtils.isEmpty(edgeIds)) {
            reverseSql.append(new StringBuilder("DELETE FROM fta_edge WHERE id IN('").append(getInSql(edgeIds)).append("');"));
        }
        if (!CollectionUtils.isEmpty(analysisIds)) {
            reverseSql.append(new StringBuilder("DELETE FROM fta_analysis WHERE analysis_id IN('").append(getInSql(analysisIds)).append("');"));
        }
        return reverseSql;
    }

    /**
     * 获取执行删除的sql，将已被逻辑删除的数据直接删掉
     *
     * @param nodeIds
     * @param edgeIds
     * @param analysisIds
     * @param operatorIds
     * @param detailIds
     * @return
     */
    public String getDeleteSql(List<String> nodeIds, List<String> edgeIds, List<String> analysisIds, List<Integer> operatorIds, List<Integer> detailIds) {
        StringBuilder reverseSql = new StringBuilder();
        if (!CollectionUtils.isEmpty(nodeIds)) {
            reverseSql.append(new StringBuilder("DELETE FROM fta_node WHERE del_flag = '1' AND id IN('").append(getInSql(nodeIds)).append("');"));
        }
//        if (!CollectionUtils.isEmpty(edgeIds)) {
//            reverseSql.append(new StringBuilder("DELETE FROM fta_edge WHERE del_flag = '1' AND id IN('").append(getInSql(edgeIds)).append("');"));
//        }
        if (!CollectionUtils.isEmpty(analysisIds)) {
            reverseSql.append(new StringBuilder("DELETE FROM fta_analysis WHERE del_flag = '1' AND analysis_id IN('").append(getInSql(analysisIds)).append("');"));
        }
        if (!CollectionUtils.isEmpty(operatorIds)) {
            reverseSql.append(new StringBuilder("DELETE FROM fta_operator WHERE id IN(").append(getInIntegerSql(operatorIds)).append(");"));
        }
        if (!CollectionUtils.isEmpty(detailIds)) {
            reverseSql.append(new StringBuilder("DELETE FROM fta_operator_detail WHERE id IN(").append(getInIntegerSql(detailIds)).append(");"));
        }
        return reverseSql.toString();
    }

    public void runSql(String sql) {
        String[] sqls = sql.split(";");
        for (int i = 0; i < sqls.length; i++) {
            if (sqls[i].contains("DELETE")) {
                com.baomidou.mybatisplus.extension.toolkit.SqlRunner.db().delete(sqls[i]);
            } else if (sqls[i].contains("UPDATE")) {
                com.baomidou.mybatisplus.extension.toolkit.SqlRunner.db().update(sqls[i]);
            }
        }
    }

//    public static void main(String[] args) {
//        String formula = "count+price*99-(600-3*15)%(((68-9)-3)*2-100)+10000%7*71";
//        System.out.println(System.currentTimeMillis());
////        Double execute = (Double) AviatorEvaluator.execute(formula);
//        System.out.println(System.currentTimeMillis());
////        System.out.println(execute);
//        FelEngine fel = new FelEngineImpl();
//        FelContext ctx = fel.getContext();
//        ctx.set("count", 1000);
//        ctx.set("price", 100);
//        System.out.println(System.currentTimeMillis());
//        Object result = fel.eval(formula);
//        System.out.println(System.currentTimeMillis());
//        System.out.println(result);
//    }
}




