// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.fta.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nationrel.modules.fta.constant.FtaConstants;
import com.nationrel.modules.fta.entity.FtaAnalysis;
import com.nationrel.modules.fta.entity.FtaProject;
import com.nationrel.modules.fta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.fta.general.domain.enums.fta.FtaParamDataExceptionEnum;
import com.nationrel.modules.fta.general.util.common.BasicDataUtil;
import com.nationrel.modules.fta.mapper.FtaProjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Objects;

@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FtaProjectService {

    private final FtaProjectMapper ftaProjectMapper;
    private final FtaAnalysisService analysisService;

    /**
     * 分页查询
     *
     * @param userInfo
     * @param projectId
     * @param projectName
     * @param startDate
     * @param endDate
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public Result<IPage<FtaProject>> listByPage(HeaderUserInfo userInfo, String projectId, String projectName, Date startDate, Date endDate, int pageIndex, int pageSize) {
        // 创建分页条件
        Page<FtaProject> page = new Page<>(pageIndex, pageSize);
        // 创建基础查询条件
        QueryWrapper<FtaProject> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(FtaProject::getCreateBy, userInfo.getUserName());
        queryWrapper.lambda().eq(FtaProject::getDelFlag, FtaConstants.DEFAULT_DEL_FLAG);
        if (StringUtils.isNotBlank(projectId)) {
            queryWrapper.lambda().like(FtaProject::getProjectId, projectId);
        }
        if (StringUtils.isNotEmpty(projectName)) {
            queryWrapper.lambda().like(FtaProject::getProjectName, projectName);
        }
        if (startDate != null) {
            queryWrapper.lambda().ge(FtaProject::getCreateTime, startDate);
        }
        if (endDate != null) {
            queryWrapper.lambda().le(FtaProject::getCreateTime, endDate);
        }
        return Result.ok(ftaProjectMapper.selectPage(page, queryWrapper));
    }

    /**
     * 添加分析数据信息
     *
     * @param userInfo
     * @param insert
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Result<String> addAnalysis(HeaderUserInfo userInfo, FtaProject insert) {

        String projectId = BasicDataUtil.uuidGenerator();
        insert.setProjectId(projectId);
        insert.setCreateTime(new Date());
        insert.setUpdateTime(new Date());
        insert.setCreateBy(userInfo.getUserName());
        insert.setUpdateBy(userInfo.getUserName());
        insert.setDelFlag(FtaConstants.DEFAULT_DEL_FLAG);
        ftaProjectMapper.insert(insert);

        // 添加分析数据
        FtaAnalysis analysis = new FtaAnalysis();
        analysis.setProjectId(projectId);
        analysis.setAnalysisName("fta-a");
        analysisService.addAnalysis(userInfo, analysis);

        return Result.ok(projectId);
    }

    /**
     * 更新修改者信息
     *
     * @param userInfo
     * @param projectId
     */
    public static void updateUpdateInfo(HeaderUserInfo userInfo, String projectId, FtaProjectMapper ftaProjectMapper) {
        ftaProjectMapper.updateUpdateInfo(new Date(), userInfo.getUserName(), projectId);
    }

    /**
     * 更新数据
     *
     * @param userInfo
     * @param entity
     * @return
     */
    public Result<String> updateAnalysisInfo(HeaderUserInfo userInfo, FtaProject entity) {
        if (StringUtils.isBlank(entity.getProjectId()) || StringUtils.isBlank(entity.getProjectId())) {
            return Result.error(FtaParamDataExceptionEnum.getLanguageMessageByKey(FtaParamDataExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        ftaProjectMapper.updateById(entity);
        updateUpdateInfo(userInfo, entity.getProjectId(), ftaProjectMapper);
        return Result.ok();
    }

    /**
     * 逻辑删除数据
     *
     * @param userInfo
     * @param altaId
     * @return
     * @throws Exception
     */
    public Result<String> deleteAnalysisInfo(HeaderUserInfo userInfo, String altaId) {
        FtaProject analysis = ftaProjectMapper.selectById(altaId);
        if (Objects.equals(userInfo.getUserName(), analysis.getCreateBy())) {
            analysis.setDelFlag(FtaConstants.DEL_FLAG);
            ftaProjectMapper.updateById(analysis);
        }
        return Result.ok();
    }
}

